@@ -1,4 +1,6 @@
 /********************************************************************\
+ * Scrub.h -- convert single-entry accounts to clean double-entry   *
+ *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
  * published by the Free Software Foundation; either version 2 of   *
@@ -44,6 +46,9 @@
  *    go into and clean up.  Kind of like the unix "Lost+Found" directory
  *    for orphaned inodes.
  *
+ * The xaccTransScrubOrphans() method scrubs only the splits in the
+ *    given transaction. A root account group must be provided.
+ *
  * The xaccAccountScrubOrphans() method performs this scrub only for the 
  *    indicated account, and not for any of its children.
  *
@@ -53,15 +58,27 @@
  * The xaccGroupScrubOrphans() method performs this scrub for the 
  *    child accounts of this group.
  */
+void xaccTransScrubOrphans (Transaction *trans, AccountGroup *root);
 void xaccAccountScrubOrphans (Account *acc);
 void xaccAccountTreeScrubOrphans (Account *acc);
 void xaccGroupScrubOrphans (AccountGroup *grp);
 
+/* The ScrubSplit methods ensure that splits with the same currency
+ *   and security have the same share quantity and value.
+ */
+void xaccSplitScrub (Split *split);
+void xaccTransScrubSplits (Transaction *trans);
+void xaccAccountScrubSplits (Account *account);
+void xaccAccountTreeScrubSplits (Account *account);
+void xaccGroupScrubSplits (AccountGroup *group);
+
 /* The xaccScrubImbalance() method searches for transactions that do
  *    not balance to zero. If any such transactions are found, a split
  *    is created to offset this amount and is added to an "imbalance"
  *    account.
  */
+void xaccTransScrubImbalance (Transaction *trans, AccountGroup *root,
+                              Account *parent);
 void xaccAccountScrubImbalance (Account *acc);
 void xaccAccountTreeScrubImbalance (Account *acc);
 void xaccGroupScrubImbalance (AccountGroup *grp);
