@@ -22,11 +22,13 @@
 \********************************************************************/
 
 #define _GNU_SOURCE
+#include "config.h"
+
 #include <assert.h>
 #include <stdio.h>
 #include <string.h>
 
-#include "config.h"
+#include <glib.h>
 
 #include "Account.h"
 #include "AccountP.h"
@@ -35,7 +37,7 @@
 #include "Transaction.h"
 #include "TransactionP.h"
 #include "TransLog.h"
-#include "util.h"
+#include "gnc-engine-util.h"
 
 /*
  * The logfiles are useful for tracing, journalling, error recovery.
@@ -100,8 +102,8 @@
 
 
 static int gen_logs = 1;
-static FILE * trans_log = 0x0;
-static char * log_base_name = 0x0;
+static FILE * trans_log = NULL;
+static char * log_base_name = NULL;
 
 /********************************************************************\
 \********************************************************************/
@@ -117,8 +119,8 @@ xaccLogSetBaseName (const char *basepath)
 {
    if (!basepath) return;
 
-   if (log_base_name) free (log_base_name);
-   log_base_name = strdup (basepath);
+   g_free (log_base_name);
+   log_base_name = g_strdup (basepath);
 
    if (trans_log) {
       xaccCloseLog();
@@ -138,28 +140,26 @@ xaccOpenLog (void)
    if (!gen_logs) return;
    if (trans_log) return;
 
-   if (!log_base_name) log_base_name = strdup ("translog");
+   if (!log_base_name) log_base_name = g_strdup ("translog");
 
    /* tag each filename with a timestamp */
    timestamp = xaccDateUtilGetStampNow ();
 
-   filename = (char *) malloc (strlen (log_base_name) + 50);
-   strcpy (filename, log_base_name);
-   strcat (filename, ".");
-   strcat (filename, timestamp);
-   strcat (filename, ".log");
+   filename = g_strconcat (log_base_name, ".", timestamp, ".log", NULL);
 
    trans_log = fopen (filename, "a");
    if (!trans_log) {
       int norr = errno;
       printf ("Error: xaccOpenLog(): cannot open journal \n"
               "\t %d %s\n", norr, strerror (norr));
-      free (filename);
-      free (timestamp);
+
+      g_free (filename);
+      g_free (timestamp);
       return;
    }
-   free (filename);
-   free (timestamp);
+
+   g_free (filename);
+   g_free (timestamp);
 
    /* use tab-separated fields */
    fprintf (trans_log, "mod	id	time_now	" \
@@ -168,7 +168,6 @@ xaccOpenLog (void)
                        "memo	action	reconciled	" \
                        "amount	price date_reconciled\n");
    fprintf (trans_log, "-----------------\n");
-
 }
 
 /********************************************************************\
@@ -180,7 +179,7 @@ xaccCloseLog (void)
    if (!trans_log) return;
    fflush (trans_log);
    fclose (trans_log);
-   trans_log = 0x0;
+   trans_log = NULL;
 }
 
 /********************************************************************\
@@ -189,8 +188,7 @@ xaccCloseLog (void)
 void
 xaccTransWriteLog (Transaction *trans, char flag)
 {
-   Split *split;
-   int i = 0;
+   GList *node;
    char *dnow, *dent, *dpost, *drecn; 
 
    if (!gen_logs) return;
@@ -202,38 +200,44 @@ xaccTransWriteLog (Transaction *trans, char flag)
 
    fprintf (trans_log, "===== START\n");
 
-   split = trans->splits[0];
-   while (split) {
-      char * accname = "";
-      if (split->acc) accname = split->acc->accountName;
+   for (node = trans->splits; node; node = node->next) {
+      Split *split = node->data;
+      const char * accname = "";
+
+      if (xaccSplitGetAccount(split))
+        accname = xaccAccountGetName (xaccSplitGetAccount(split));
+
       drecn = xaccDateUtilGetStamp (split->date_reconciled.tv_sec);
 
       /* use tab-separated fields */
-      fprintf (trans_log, "%c	%p/%p	%s	%s	%s	%s	%s	" \
-               "%s	%s	%s	%c	%10.6f	%10.6f	%s\n",
+      fprintf (trans_log,
+               "%c\t%p/%p\t%s\t%s\t%s\t%s\t%s\t"
+               "%s\t%s\t%s\t%c\t%lld/%lld\t%lld/%lld\t%s\n",
                flag,
                trans, split,  /* trans+split make up unique id */
-               dnow,
-               dent, 
-               dpost, 
-               accname,
-               trans->num, 
-               trans->description,
-               split->memo,
-               split->action,
+               dnow ? dnow : "",
+               dent ? dent : "", 
+               dpost ? dpost : "", 
+               accname ? accname : "",
+               trans->num ? trans->num : "", 
+               trans->description ? trans->description : "",
+               split->memo ? split->memo : "",
+               split->action ? split->action : "",
                split->reconciled,
-               split->damount,
-               split->share_price,
-               drecn
-               );
-      free (drecn);
-      i++;
-      split = trans->splits[i];
+               (long long int) gnc_numeric_num(split->damount), 
+               (long long int) gnc_numeric_denom(split->damount),
+               (long long int) gnc_numeric_num(split->value), 
+               (long long int) gnc_numeric_denom(split->value),
+               drecn ? drecn : "");
+
+      g_free (drecn);
    }
+
    fprintf (trans_log, "===== END\n");
-   free (dnow);
-   free (dent);
-   free (dpost);
+
+   g_free (dnow);
+   g_free (dent);
+   g_free (dpost);
 
    /* get data out to the disk */
    fflush (trans_log);
@@ -255,7 +259,7 @@ xaccSplitAsString(Split *split, const char prefix[]) {
   size_t result_size;
   FILE *stream = open_memstream(&result, &result_size); 
   const char *split_memo = xaccSplitGetMemo(split);
-  const double split_value = xaccSplitGetValue(split);
+  const double split_value = DxaccSplitGetValue(split);
   Account *split_dest = xaccSplitGetAccount(split);
   const char *dest_name =
     split_dest ? xaccAccountGetName(split_dest) : NULL;
@@ -272,6 +276,22 @@ xaccSplitAsString(Split *split, const char prefix[]) {
   return(result);
 }
 
+static char *
+xaccTransGetDateStr (Transaction *trans)
+{
+   char buf [MAX_DATE_LENGTH];
+   struct tm *date;
+   time_t secs;
+
+   secs = xaccTransGetDate (trans);
+
+   date = localtime (&secs);
+
+   printDate(buf, date->tm_mday, date->tm_mon+1, date->tm_year +1900);
+
+   return g_strdup (buf);
+}
+
 char *
 xaccTransAsString(Transaction *txn, const char prefix[]) {
   char *result = NULL;
@@ -281,7 +301,7 @@ xaccTransAsString(Transaction *txn, const char prefix[]) {
   const char *num = xaccTransGetNum(txn);
   const char *desc = xaccTransGetDescription(txn);
   const char *memo = xaccSplitGetMemo(xaccTransGetSplit(txn, 0));
-  const double total = xaccSplitGetValue(xaccTransGetSplit(txn, 0));
+  const double total = DxaccSplitGetValue(xaccTransGetSplit(txn, 0));
   
   assert(stream);
 
