@@ -128,6 +128,10 @@ struct _split
   gnc_numeric  share_balance;
   gnc_numeric  share_cleared_balance;
   gnc_numeric  share_reconciled_balance;
+
+  /* -------------------------------------------------------------- */
+  /* Backend private expansion data */
+  guint32  idata;     /* used by the sql backend for kvp management */
 };
 
 
@@ -193,6 +197,10 @@ struct _transaction
    * any changes made if/when the edit is abandoned.
    */
   Transaction *orig;
+
+  /* -------------------------------------------------------------- */
+  /* Backend private expansion data */
+  guint32  idata;     /* used by the sql backend for kvp management */
 };
 
 /* Set the transaction's GUID. This should only be done when reading
