@@ -24,27 +24,68 @@
  * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
-#include "top-level.h"
+#include "config.h"
 
 #include <gnome.h>
 
-#include "gnucash.h"
-#include "messages.h"
 #include "AccWindow.h"
 #include "FileDialog.h"
-#include "window-main.h"
+#include "account-tree.h"
 #include "dialog-utils.h"
-#include "account-treeP.h"
-#include "util.h"
+#include "gnc-component-manager.h"
+#include "gnc-engine-util.h"
+#include "gnc-ui-util.h"
+#include "gnucash.h"
+#include "messages.h"
+#include "window-main.h"
+
+#define ACCOUNT_TREE_CM_CLASS "account-tree"
+
+/* Signal codes */
+enum
+{
+  SELECT_ACCOUNT,
+  UNSELECT_ACCOUNT,
+  ACTIVATE_ACCOUNT,
+  LAST_SIGNAL
+};
 
 
+/** Static Globals ****************************************************/
 static GtkCTreeClass *parent_class = NULL;
 static guint account_tree_signals[LAST_SIGNAL];
 
-static GSList *account_trees = NULL;
+/* This static indicates the debugging module that this .o belongs to.  */
+static short module = MOD_GUI;
+
+
+/** Static function declarations **************************************/
+static void gnc_account_tree_init(GNCAccountTree *tree);
+static void gnc_account_tree_class_init(GNCAccountTreeClass *klass);
+static gint gnc_account_tree_key_press(GtkWidget *widget, GdkEventKey *event);
+static gint gnc_account_tree_button_press(GtkWidget *widget,
+ 					  GdkEventButton *event);
+static void gnc_account_tree_select_row(GtkCTree *ctree,
+					GtkCTreeNode *row,
+					gint column);
+static void gnc_account_tree_unselect_row(GtkCTree *ctree,
+					  GtkCTreeNode *row,
+					  gint column);
+static GtkCTreeNode * gnc_account_tree_insert_row(GNCAccountTree *tree,
+						  GtkCTreeNode *parent,
+						  GtkCTreeNode *sibling,
+						  Account *acc);
+static void gnc_account_tree_fill(GNCAccountTree *tree,
+                                  GHashTable *expanded_accounts,
+				  GtkCTreeNode *parent,
+				  AccountGroup *accts);
+static void gnc_account_tree_set_view_info_real(GNCAccountTree *tree);
+static void gnc_account_tree_update_column_visibility (GNCAccountTree *tree);
+static void gnc_account_tree_destroy(GtkObject *object);
+
 
 GtkType
-gnc_account_tree_get_type()
+gnc_account_tree_get_type (void)
 {
   static GtkType gnc_account_tree_type = 0;
 
@@ -70,6 +111,14 @@ gnc_account_tree_get_type()
 }
 
 
+static void
+refresh_handler (GHashTable *changes, gpointer user_data)
+{
+  GNCAccountTree *tree = user_data;
+
+  gnc_account_tree_refresh (tree);
+}
+
 /********************************************************************\
  * gnc_account_tree_new                                             *
  *   creates the account tree                                       *
@@ -77,13 +126,20 @@ gnc_account_tree_get_type()
  * Returns: the account tree widget, or NULL if there was a problem.*
 \********************************************************************/
 GtkWidget *
-gnc_account_tree_new()
+gnc_account_tree_new (void)
 {
   GtkWidget *tree;
+  gint component_id;
 
   tree = GTK_WIDGET(gtk_type_new(gnc_account_tree_get_type()));
 
-  account_trees = g_slist_prepend(account_trees, tree);
+  component_id = gnc_register_gui_component (ACCOUNT_TREE_CM_CLASS,
+                                             refresh_handler, NULL,
+                                             tree);
+
+  gnc_gui_component_watch_entity_type (component_id,
+                                       GNC_ID_ACCOUNT,
+                                       GNC_EVENT_MODIFY | GNC_EVENT_DESTROY);
 
   return tree;
 }
@@ -99,26 +155,29 @@ gnc_account_tree_new()
  *                                                                  *
  * Returns: the account tree widget, or NULL if there was a problem.*
 \********************************************************************/
+
 GtkWidget *
-gnc_account_tree_new_with_root(Account * root)
+gnc_account_tree_new_with_root (Account * root)
 {
   GNCAccountTree *tree;
 
-  tree = GNC_ACCOUNT_TREE(gnc_account_tree_new());
-  tree->root_account = root;
+  tree = GNC_ACCOUNT_TREE (gnc_account_tree_new ());
+  tree->root_account = *xaccAccountGetGUID (root);
 
-  return GTK_WIDGET(tree);
+  return GTK_WIDGET (tree);
 }
 
 
 static void
-gnc_account_tree_init(GNCAccountTree *tree)
+gnc_account_tree_init (GNCAccountTree *tree)
 {
-  tree->root_account     = NULL;
+  tree->root_account     = *xaccGUIDNULL ();
   tree->current_accounts = NULL;
-  tree->ignore_unselect  = GNC_F;
-  tree->filter           = NULL;
-  tree->filter_data      = NULL;
+  tree->ignore_unselect  = FALSE;
+  tree->view_filter      = NULL;
+  tree->view_filter_data = NULL;
+  tree->selectable_filter = NULL;
+  tree->selectable_filter_data = NULL;
 
   gnc_init_account_view_info(&tree->avi);
 
@@ -163,7 +222,7 @@ gnc_account_tree_init(GNCAccountTree *tree)
 
   tree->deficit_style = NULL;
 
-#if !USE_NO_COLOR
+  if (gnc_color_deficits())
   {
     GdkColormap *cm = gtk_widget_get_colormap(GTK_WIDGET(tree));
     GtkStyle *style = gtk_widget_get_style(GTK_WIDGET(tree));
@@ -175,7 +234,6 @@ gnc_account_tree_init(GNCAccountTree *tree)
 
     gdk_colormap_alloc_color(cm, &style->fg[GTK_STATE_NORMAL], FALSE, TRUE);
   }
-#endif
 }
 
 static void
@@ -292,6 +350,9 @@ gnc_account_tree_refresh(GNCAccountTree * tree)
   GList         *current_accounts;
   GtkAdjustment *adjustment;
   gfloat         save_value = 0.0;
+  Account       *root_account;
+
+  ENTER(" ");
 
   adjustment = gtk_clist_get_vadjustment(GTK_CLIST(tree));
   if (adjustment != NULL)
@@ -305,9 +366,11 @@ gnc_account_tree_refresh(GNCAccountTree * tree)
 
   gtk_clist_clear(clist);
 
+  root_account = xaccAccountLookup (&tree->root_account);
+
   gnc_account_tree_fill(tree, expanded_accounts,
 			gnc_account_tree_insert_row(tree, NULL, NULL,
-						    tree->root_account),
+						    root_account),
 			gncGetCurrentGroup());
 
   gtk_clist_columns_autosize(clist);
@@ -327,26 +390,7 @@ gnc_account_tree_refresh(GNCAccountTree * tree)
 
   g_hash_table_destroy(expanded_accounts);
   g_list_free(current_accounts);
-}
-
-
-static void
-refresh_helper(gpointer tree, gpointer unused)
-{
-  gnc_account_tree_refresh(GNC_ACCOUNT_TREE(tree));
-}
-
-/********************************************************************\
- * gnc_account_tree_refresh_all                                     *
- *   refreshes the account tree                                     *
- *                                                                  *
- * Args: none                                                       *
- * Returns: nothing                                                 *
-\********************************************************************/
-void
-gnc_account_tree_refresh_all()
-{
-  g_slist_foreach(account_trees, refresh_helper, NULL);
+  LEAVE(" ");
 }
 
 
@@ -420,8 +464,8 @@ gnc_account_tree_expand_account(GNCAccountTree *tree, Account *account)
  * gnc_account_tree_toggle_account_expansion                        *
  *   toggle the expansion status for the given account              *
  *                                                                  *
- * Args: tree   - the tree to toggle expansion status for           *
- *       accout - the account to toggle expansion status            *
+ * Args: tree    - the tree to toggle expansion status for          *
+ *       account - the account to toggle expansion status           *
  * Returns: nothing                                                 *
 \********************************************************************/
 void
@@ -445,6 +489,27 @@ gnc_account_tree_toggle_account_expansion (GNCAccountTree *tree,
 }
 
 
+/********************************************************************\
+ * gnc_account_tree_expand_all                                      *
+ *   fully expand the tree                                          *
+ *                                                                  *
+ * Args: tree   - the tree to fully expand                          *
+ * Returns: nothing                                                 *
+\********************************************************************/
+void
+gnc_account_tree_expand_all (GNCAccountTree *tree)
+{
+  GtkCTree *ctree;
+
+  g_return_if_fail (tree != NULL);
+  g_return_if_fail (IS_GNC_ACCOUNT_TREE(tree));
+
+  ctree = GTK_CTREE(tree);
+
+  gtk_ctree_expand_recursive (ctree, NULL);
+}
+
+
 /********************************************************************\
  * gnc_account_tree_select_account                                  *
  *   select an account in the tree and expands the tree to make     *
@@ -493,6 +558,129 @@ gnc_account_tree_select_account(GNCAccountTree *tree,
 }
 
 
+/********************************************************************\
+ * gnc_account_tree_select_subaccounts                              *
+ *   select the account and all subaccounts of an account that are  *
+ *   in the tree and expands the tree to ensure they are visible.   *
+ *   It may also scroll the tree to ensure it is visible.           *
+ *                                                                  *
+ * Args: tree    - tree to be modified                              *
+ *       account - account whose subaccountn are to be selected     *
+ *       show    - if true, scroll the tree                         *
+ * Returns: true if the account was found                           *
+\********************************************************************/
+gboolean
+gnc_account_tree_select_subaccounts (GNCAccountTree *tree,
+                                     Account *account,
+                                     gboolean show)
+{
+  GtkCTree *ctree = GTK_CTREE(tree);
+  GtkCTreeNode *node, *n;
+  GtkCTreeRow  *row;
+
+  /* Get the node with the account */
+  node = gtk_ctree_find_by_row_data (ctree, NULL, account);
+
+  if (node == NULL)
+    return FALSE;
+
+  /* Expand all the parents */
+  row = GTK_CTREE_ROW(node);
+  while ((n = row->parent) != NULL)
+  {
+    gtk_ctree_expand (ctree, n);
+    row = GTK_CTREE_ROW(n);
+  }
+
+  /* Expand them & select them */
+  gtk_ctree_expand_recursive (ctree, node);
+  gtk_ctree_select_recursive (ctree, node);
+
+  if (!show)
+    return TRUE;
+
+  /* Make sure it's visible */
+  if (gtk_ctree_node_is_visible (ctree, node) != GTK_VISIBILITY_FULL)
+    gtk_ctree_node_moveto (ctree, node, 0, 0.5, 0.0);
+
+  return TRUE;
+}
+
+
+/********************************************************************\
+ * gnc_account_tree_unselect_account                                *
+ *   unselect an account in the tree                                *
+ *                                                                  *
+ * Args: tree    - tree to be modified                              *
+ *       account - account to be unselected                         *
+ *       show    - if true, scroll the tree                         *
+ * Returns: true if the account was found                           *
+\********************************************************************/
+gboolean
+gnc_account_tree_unselect_account (GNCAccountTree *tree,
+                                   Account        *account,
+                                   gboolean       show)
+{
+  GtkCTree *ctree = GTK_CTREE(tree);
+  GtkCTreeNode *node;
+
+  /* Get the node with the account */
+  node = gtk_ctree_find_by_row_data(ctree, NULL, account);
+
+  if (node == NULL)
+    return FALSE;
+
+  /* unselect it */
+  gtk_ctree_unselect(ctree, node);
+
+  if (!show)
+    return TRUE;
+
+  /* Make sure it's visible */
+  if (gtk_ctree_node_is_visible(ctree, node) != GTK_VISIBILITY_FULL)
+    gtk_ctree_node_moveto(ctree, node, 0, 0.5, 0.0);
+
+  return TRUE;
+}
+
+
+/********************************************************************\
+ * gnc_account_tree_unselect_subaccounts                            *
+ *   unselect an account and all its subaccounts in the tree        *
+ *                                                                  *
+ * Args: tree    - tree to be modified                              *
+ *       account - account to be unselected recursively             *
+ *       show    - if true, scroll the tree                         *
+ * Returns: true if the account was found                           *
+\********************************************************************/
+gboolean
+gnc_account_tree_unselect_subaccounts (GNCAccountTree *tree,
+                                       Account *account,
+                                       gboolean show)
+{
+  GtkCTree *ctree = GTK_CTREE(tree);
+  GtkCTreeNode *node;
+
+  /* Get the node with the account */
+  node = gtk_ctree_find_by_row_data(ctree, NULL, account);
+
+  if (node == NULL)
+    return FALSE;
+
+  /* unselect it */
+  gtk_ctree_unselect_recursive (ctree, node);
+
+  if (!show)
+    return TRUE;
+
+  /* Make sure it's visible */
+  if (gtk_ctree_node_is_visible(ctree, node) != GTK_VISIBILITY_FULL)
+    gtk_ctree_node_moveto(ctree, node, 0, 0.5, 0.0);
+
+  return TRUE;
+}
+
+
 /********************************************************************\
  * gnc_account_tree_select_accounts                                 *
  *   select a list of accounts in the tree, expanding the parents   *
@@ -532,91 +720,34 @@ gnc_account_tree_select_accounts(GNCAccountTree *tree,
 
 
 /********************************************************************\
- * gnc_account_tree_remove_account                                  *
- *   removes an account from the tree                               *
- *                                                                  *
- * Args: tree - tree to be modified                                 *
- *       account - account to be inserted                           *
- * Returns: nothing                                                 *
-\********************************************************************/
-void
-gnc_account_tree_remove_account(GNCAccountTree *tree, Account *account)
-{
-  GtkCTreeNode *node;
-
-  node = gtk_ctree_find_by_row_data(GTK_CTREE(tree),
-				    NULL, account);
-
-  if (node != NULL)
-    gtk_ctree_remove_node(GTK_CTREE(tree), node);
-}
-
-
-static void
-remove_helper(gpointer tree, gpointer account)
-{
-  gnc_account_tree_remove_account(GNC_ACCOUNT_TREE(tree), account);
-}
-
-/********************************************************************\
- * gnc_account_tree_remove_account_all                              *
- *   removes an account from all account trees                      *
- *                                                                  *
- * Args: account - account to be inserted                           *
- * Returns: nothing                                                 *
-\********************************************************************/
-void
-gnc_account_tree_remove_account_all(Account *account)
-{
-  g_slist_foreach(account_trees, remove_helper, account);
-}
-
-
-/********************************************************************\
- * gnc_account_tree_insert_account                                  *
- *   inserts a new account into the tree                            *
- *                                                                  *
- * Args: tree - tree to insert account                              *
- *       account - account to be inserted                           *
- * Returns: nothing                                                 *
-\********************************************************************/
-void
-gnc_account_tree_insert_account(GNCAccountTree *tree, Account *account)
-{
-  /* for now, just punt. Maybe we'll do it faster later. */
-  gnc_account_tree_refresh(tree);
-}
-
-
-/********************************************************************\
- * gnc_account_tree_show_categories                                 *
+ * gnc_account_tree_show_income_expense                             *
  *   shows the income/expense accounts in a tree                    *
  *                                                                  *
  * Args: tree - tree to show income/expense accounts                *
  * Returns: nothing                                                 *
 \********************************************************************/
 void
-gnc_account_tree_show_categories(GNCAccountTree *tree)
+gnc_account_tree_show_income_expense(GNCAccountTree *tree)
 {
-  tree->avi.include_type[EXPENSE] = GNC_T;
-  tree->avi.include_type[INCOME] = GNC_T;
+  tree->avi.include_type[EXPENSE] = TRUE;
+  tree->avi.include_type[INCOME] = TRUE;
 
   gnc_account_tree_refresh(tree);
 }
 
 
 /********************************************************************\
- * gnc_account_tree_hide_categories                                 *
+ * gnc_account_tree_hide_income_expense                             *
  *   hides the income/expense accounts in a tree                    *
  *                                                                  *
  * Args: tree - tree to hide income/expense accounts                *
  * Returns: nothing                                                 *
 \********************************************************************/
 void
-gnc_account_tree_hide_categories(GNCAccountTree *tree)
+gnc_account_tree_hide_income_expense(GNCAccountTree *tree)
 {
-  tree->avi.include_type[EXPENSE] = GNC_F;
-  tree->avi.include_type[INCOME] = GNC_F;
+  tree->avi.include_type[EXPENSE] = FALSE;
+  tree->avi.include_type[INCOME] = FALSE;
 
   gnc_account_tree_refresh(tree);
 }
@@ -689,6 +820,56 @@ gnc_account_tree_get_current_accounts (GNCAccountTree *tree)
 }
 
 
+/********************************************************************\
+ * gnc_account_tree_get_focus_account                               *
+ *   return the account at the focus row, or NULL if no focus       *
+ *                                                                  *
+ * Args: tree - tree to get focus account from                      *
+ * Returns: Account at focus row or NULL                            *
+\********************************************************************/
+Account *
+gnc_account_tree_get_focus_account (GNCAccountTree *tree)
+{
+  GtkCTree *ctree;
+  GtkCTreeNode *node;
+  gint row;
+
+  if (!tree)
+    return NULL;
+
+  ctree = GTK_CTREE (tree);
+
+  row = GTK_CLIST(tree)->focus_row;
+  if (row < 0)
+    return NULL;
+
+  node = gtk_ctree_node_nth (ctree, row);
+  if (!node)
+    return NULL;
+
+  return gtk_ctree_node_get_row_data (ctree, node);
+}
+
+
+/********************************************************************\
+ * gnc_account_tree_account_selected                                *
+ *   return TRUE if the account is selected                         *
+ *                                                                  *
+ * Args: tree    - tree to check selection for                      *
+ *       account - account to check selection for                   *
+ * Returns: TRUE if account is selected                             *
+\********************************************************************/
+gboolean
+gnc_account_tree_account_selected (GNCAccountTree *tree,
+                                   Account *account)
+{
+  if (tree == NULL)
+    return FALSE;
+
+  return g_list_find (tree->current_accounts, account) != NULL;
+}
+
+
 /********************************************************************\
  * gnc_init_account_view_info                                       *
  *   initialize an account view info structure with default values  *
@@ -702,19 +883,19 @@ gnc_init_account_view_info(AccountViewInfo *avi)
   int i;
 
   for (i = 0; i < NUM_ACCOUNT_TYPES; i++)
-    avi->include_type[i] = GNC_T;
+    avi->include_type[i] = TRUE;
 
   for (i = 0; i < NUM_ACCOUNT_FIELDS; i++)
-    avi->show_field[i] = GNC_F;
+    avi->show_field[i] = FALSE;
 
-  avi->show_field[ACCOUNT_NAME] = GNC_T;
-  avi->show_field[ACCOUNT_DESCRIPTION] = GNC_T;
-  avi->show_field[ACCOUNT_TOTAL] = GNC_T;
+  avi->show_field[ACCOUNT_NAME] = TRUE;
+  avi->show_field[ACCOUNT_DESCRIPTION] = TRUE;
+  avi->show_field[ACCOUNT_TOTAL] = TRUE;
 }
 
 /********************************************************************\
- * gnc_account_tree_set_filter                                      *
- *   sets the account filter to use with the tree                   *
+ * gnc_account_tree_set_view_filter                                 *
+ *   sets the account view filter to use with the tree              *
  *                                                                  *
  * Args: tree      - the tree to set the filter on                  *
  *       filter    - the filter function to use                     *
@@ -722,15 +903,36 @@ gnc_init_account_view_info(AccountViewInfo *avi)
  * Returns: nothing                                                 *
 \********************************************************************/
 void
-gnc_account_tree_set_filter(GNCAccountTree *tree,
-                            AccountFilter filter,
-                            gpointer user_data)
+gnc_account_tree_set_view_filter(GNCAccountTree *tree,
+                                 AccountFilter filter,
+                                 gpointer user_data)
 {
   g_return_if_fail(tree != NULL);
   g_return_if_fail(IS_GNC_ACCOUNT_TREE(tree));
 
-  tree->filter = filter;
-  tree->filter_data = user_data;
+  tree->view_filter = filter;
+  tree->view_filter_data = user_data;
+}
+
+/********************************************************************\
+ * gnc_account_tree_set_selectable_filter                           *
+ *   sets the account selectable filter to use with the tree        *
+ *                                                                  *
+ * Args: tree      - the tree to set the filter on                  *
+ *       filter    - the filter function to use                     *
+ *       user_data - the user_data for the callback                 *
+ * Returns: nothing                                                 *
+\********************************************************************/
+void
+gnc_account_tree_set_selectable_filter (GNCAccountTree *tree,
+                                        AccountFilter filter,
+                                        gpointer user_data)
+{
+  g_return_if_fail(tree != NULL);
+  g_return_if_fail(IS_GNC_ACCOUNT_TREE(tree));
+
+  tree->selectable_filter = filter;
+  tree->selectable_filter_data = user_data;
 }
 
 static void
@@ -754,12 +956,13 @@ gnc_account_tree_set_view_info_real(GNCAccountTree *tree)
   tree->column_fields[i++] = ACCOUNT_TOTAL_EURO;
 
   tree->column_fields[i++] = ACCOUNT_NOTES;
+  tree->column_fields[i++] = ACCOUNT_TAX_INFO;
 
   tree->num_columns = i;
 
   for (i = 0; i < tree->num_columns; i++)
     tree->column_headings[i] =
-      gnc_ui_get_account_field_name(tree->column_fields[i]);
+      gnc_ui_account_get_field_name (tree->column_fields[i]);
 }
 
 static gint
@@ -806,7 +1009,7 @@ gnc_account_tree_button_press(GtkWidget *widget,
       node = gtk_ctree_node_nth(ctree, row);
       account = gtk_ctree_node_get_row_data(ctree, node);
 
-      GNC_ACCOUNT_TREE(ctree)->ignore_unselect = GNC_T;
+      GNC_ACCOUNT_TREE(ctree)->ignore_unselect = TRUE;
 
       gtk_signal_emit(GTK_OBJECT(widget),
 		      account_tree_signals[ACTIVATE_ACCOUNT],
@@ -831,7 +1034,8 @@ gnc_account_tree_select_row(GtkCTree *ctree,
   Account *account;
   GList *node;
 
-  tree->ignore_unselect = GNC_F;
+  tree->ignore_unselect = FALSE;
+
   account = gtk_ctree_node_get_row_data(ctree, GTK_CTREE_NODE(row));
 
   if (account != NULL)
@@ -860,7 +1064,7 @@ gnc_account_tree_unselect_row(GtkCTree *ctree,
 
   if (tree->ignore_unselect)
   {
-    tree->ignore_unselect = GNC_F;
+    tree->ignore_unselect = FALSE;
     return;
   }
 
@@ -888,22 +1092,23 @@ gnc_account_tree_fill(GNCAccountTree *tree,
 		      GtkCTreeNode   *parent,
                       AccountGroup   *accts)
 {
-  Account *account;
   AccountGroup *acc_children;
+  GNCAccountType type;
   GtkCTreeNode *node;
-  gint totalAccounts = xaccGroupGetNumAccounts(accts);
-  gint currentAccount;
-  gint type;
+  GList *list;
+  GList *n;
+
+  ENTER ("grp=%p", accts);
+
+  list = xaccGroupGetAccountList (accts);
 
   /* Add each account to the tree */  
-  for ( currentAccount = 0;
-        currentAccount < totalAccounts;
-        currentAccount++ )
+  for (n = list; n; n = n->next)
   {
-    account = xaccGroupGetAccount(accts, currentAccount);
+    Account *account = n->data;
 
-    if (tree->filter != NULL)
-      if (!tree->filter(account, tree->filter_data))
+    if (tree->view_filter != NULL)
+      if (!tree->view_filter(account, tree->view_filter_data))
         continue;
 
     type = xaccAccountGetType(account);
@@ -913,16 +1118,26 @@ gnc_account_tree_fill(GNCAccountTree *tree,
 
     node = gnc_account_tree_insert_row(tree, parent, NULL, account);
 
+    if (tree->selectable_filter != NULL)
+    {
+      gboolean selectable;
+
+      selectable = tree->selectable_filter (account,
+                                            tree->selectable_filter_data);
+
+      gtk_ctree_node_set_selectable (GTK_CTREE(tree), node, selectable);
+    }
+
     if (g_hash_table_lookup(expanded_accounts, account) != NULL)
       gtk_ctree_expand(GTK_CTREE(tree), node);
 
     /* If this account has children,
-     * then we need to build a subtree and fill it.
-     */
+     * then we need to build a subtree and fill it. */
     acc_children = xaccAccountGetChildren(account);
     if (xaccAccountGetChildren(account) != NULL)
       gnc_account_tree_fill(tree, expanded_accounts, node, acc_children);
   }
+  LEAVE(" ");
 }
 
 static GtkCTreeNode *
@@ -940,9 +1155,18 @@ gnc_account_tree_insert_row(GNCAccountTree *tree,
 
   for (i = 0; i < tree->num_columns; i++)
   {
-    text[i] =
-      g_strdup(gnc_ui_get_account_field_value_string(acc,
-                                                     tree->column_fields[i]));
+    AccountFieldCode field;
+
+    field = tree->column_fields[i];
+
+    if (tree->avi.show_field[field])
+      text[i] = gnc_ui_account_get_field_value_string(acc, field);
+    else
+      text[i] = NULL;
+
+    /* Since string fields like notes can be NULL */
+    if (!text[i])
+      text[i] = g_strdup("");
   }
 
   text[tree->num_columns] = NULL;
@@ -954,14 +1178,14 @@ gnc_account_tree_insert_row(GNCAccountTree *tree,
   for (i = 0; i < tree->num_columns; i++)
     g_free(text[i]);
 
-#if !USE_NO_COLOR
+  if (gnc_color_deficits())
   {
     GtkStyle *style;
-    double balance;
+    gnc_numeric balance;
     gboolean deficit;
 
-    balance = gnc_ui_account_get_balance(acc, FALSE);
-    deficit = (balance < 0) && !DEQ(balance, 0);
+    balance = gnc_ui_account_get_balance (acc, FALSE);
+    deficit = gnc_numeric_negative_p (balance);
 
     if (deficit)
       style = tree->deficit_style;
@@ -973,7 +1197,7 @@ gnc_account_tree_insert_row(GNCAccountTree *tree,
 				    tree->balance_column, style);
 
     balance = gnc_ui_account_get_balance(acc, TRUE);
-    deficit = (balance < 0) && !DEQ(balance, 0);
+    deficit = gnc_numeric_negative_p (balance);
 
     if (deficit)
       style = tree->deficit_style;
@@ -988,7 +1212,6 @@ gnc_account_tree_insert_row(GNCAccountTree *tree,
 				    tree->total_column + 1, style);
     }
   }
-#endif
 
   /* Set the user_data for the tree item to the account it */
   /* represents.                                           */
@@ -1002,7 +1225,7 @@ gnc_account_tree_destroy(GtkObject *object)
 {
   GNCAccountTree *tree = GNC_ACCOUNT_TREE(object);
 
-  account_trees = g_slist_remove(account_trees, tree);
+  gnc_unregister_gui_component_by_data (ACCOUNT_TREE_CM_CLASS, tree);
 
   if (tree->deficit_style != NULL)
   {
