@@ -28,7 +28,8 @@
 #include "gnc-datedelta.h"
 #include "guile-util.h"
 #include "messages.h"
-#include "util.h"
+#include "gnc-engine-util.h"
+#include "gnc-ui-util.h"
 
 
 /** Datatypes ***********************************************************/
@@ -238,7 +239,7 @@ string_to_description(char *string, gboolean no_blank)
   if ((string == NULL) || (*string == '\0'))
   {
     if (no_blank)
-      return g_strconcat("<", NO_DESC_STR, ">", NULL);
+      return g_strconcat("<", _("No description"), ">", NULL);
     else
       return g_strdup("");
   }
@@ -340,6 +341,7 @@ static void
 load_subentry(BudgetDialog *bd)
 {
   char *string;
+  const char *const_string;
   SCM subentry;
 
   subentry = bd->current_subentry;
@@ -361,8 +363,8 @@ load_subentry(BudgetDialog *bd)
 
     value = gh_call1(getters.subentry_amount, subentry);
     amount = gh_scm2double(value);
-    string = xaccPrintAmount(amount, PRTSEP, NULL);
-    gtk_entry_set_text(GTK_ENTRY(bd->subentry_amount_entry), string);
+    const_string = DxaccPrintAmount(amount, gnc_default_print_info (FALSE));
+    gtk_entry_set_text(GTK_ENTRY(bd->subentry_amount_entry), const_string);
 
     value = gh_call1(getters.subentry_period, subentry);
     period = gh_scm2int(value);
@@ -811,6 +813,8 @@ entry_down_button_clicked(GtkButton *button, BudgetDialog *bd)
   if (sibling == NULL)
     return;
 
+  sibling = get_next_sibling_node(bd, sibling);
+
   gtk_ctree_move(ctree, node, parent, sibling);
   if (gtk_ctree_node_is_visible(ctree, node) != GTK_VISIBILITY_FULL)
     gtk_ctree_node_moveto(ctree, node, 0, 0.5, 0);
@@ -853,7 +857,7 @@ subentry_amount_entry_focus_out(GtkWidget *widget, GdkEventFocus *event,
                                 BudgetDialog *bd)
 {
   GtkEntry *entry = GTK_ENTRY(widget);
-  gchar *new_string;
+  const gchar *new_string;
   gchar *string;
   double value;
 
@@ -862,9 +866,10 @@ subentry_amount_entry_focus_out(GtkWidget *widget, GdkEventFocus *event,
   if ((string == NULL) || (*string == '\0'))
     return FALSE;
 
-  value = xaccParseAmount(string, GNC_T);
+  value = 0.0;
+  DxaccParseAmount(string, TRUE, &value, NULL);
 
-  new_string = xaccPrintAmount(value, PRTSEP, NULL);
+  new_string = DxaccPrintAmount(value, gnc_default_print_info (FALSE));
 
   if (safe_strcmp(string, new_string) == 0)
     return FALSE;
