@@ -20,14 +20,23 @@
  * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
-#ifndef __WINDOW_HELP_H__
-#define __WINDOW_HELP_H__
+#ifndef __GNC_HELP_WINDOW_H__
+#define __GNC_HELP_WINDOW_H__
 
+#include <gnome.h>
+
+#include "gnc-html.h"
+
+typedef struct _gnc_help_window gnc_help_window;
 
 /** PROTOTYPES ******************************************************/
 
-void helpWindow(GtkWidget *parent, const char *title, const char *htmlfile);
+void helpWindow(GtkWidget *parent, const char *title, const char * htmlfile);
 
-void gnc_ui_destroy_help_windows();
+gnc_help_window  * gnc_help_window_new(void);
+void             gnc_help_window_destroy(gnc_help_window * help);
+void             gnc_help_window_show_help(gnc_help_window * hw, 
+                                           const gchar * loc,
+                                           const gchar * label);
 
 #endif
