@@ -25,7 +25,9 @@
 #include "global-options.h"
 #include "option-util.h"
 #include "guile-util.h"
-#include "util.h"
+#include "gnc-engine-util.h"
+#include "gnc-ui-util.h"
+
 
 /* This static indicates the debugging module that this .o belongs to.  */
 static short module = MOD_GUI;
@@ -41,7 +43,7 @@ static GNCOptionDB *global_options = NULL;
  * Returns: nothing                                                 *
 \********************************************************************/
 void
-gnc_options_init()
+gnc_options_init(void)
 {
   SCM func = gh_eval_str("gnc:send-global-options");
   SCM options;
@@ -66,7 +68,7 @@ gnc_options_init()
  * Returns: nothing                                                 *
 \********************************************************************/
 void
-gnc_options_shutdown()
+gnc_options_shutdown(void)
 {
   gnc_option_db_destroy(global_options);
   global_options = NULL;
@@ -141,6 +143,24 @@ gnc_get_option_by_SCM(SCM guile_option)
 }
 
 
+/********************************************************************\
+ * gnc_lookup_option                                                *
+ *   looks up an option. If present, returns its SCM value,         *
+ *   otherwise returns the default.                                 *
+ *                                                                  *
+ * Args: section - section name of option                           *
+ *       name    - name of option                                   *
+ *       default - default value if not found                       *
+ * Return: option value                                             *
+\********************************************************************/
+SCM
+gnc_lookup_option(const char *section, const char *name, SCM default_value)
+{
+  return gnc_option_db_lookup_option(global_options, section,
+                                     name, default_value);
+}
+
+
 /********************************************************************\
  * gnc_lookup_boolean_option                                        *
  *   looks up a boolean option. If present, returns its value,      *
@@ -173,13 +193,33 @@ gnc_lookup_boolean_option(const char *section, const char *name,
 \********************************************************************/
 char *
 gnc_lookup_string_option(const char *section, const char *name,
-                         char *default_value)
+                         const char *default_value)
 {
   return gnc_option_db_lookup_string_option(global_options, section,
                                             name, default_value);
 }
 
 
+/********************************************************************\
+ * gnc_lookup_font_option                                           *
+ *   looks up a font option. If present, returns its malloc'ed      *
+ *   string value, otherwise returns the strdup'ed default, or NULL *
+ *   if default was NULL.                                           *
+ *                                                                  *
+ * Args: section - section name of option                           *
+ *       name    - name of option                                   *
+ *       default - default value if not found                       *
+ * Return: char * option value                                      *
+\********************************************************************/
+char *
+gnc_lookup_font_option(const char *section, const char *name,
+                       const char *default_value)
+{
+  return gnc_option_db_lookup_font_option(global_options, section,
+                                          name, default_value);
+}
+
+
 /********************************************************************\
  * gnc_lookup_multichoice_option                                    *
  *   looks up a multichoice option. If present, returns its         *
@@ -194,7 +234,7 @@ gnc_lookup_string_option(const char *section, const char *name,
 \********************************************************************/
 char *
 gnc_lookup_multichoice_option(const char *section, const char *name,
-                              char *default_value)
+                              const char *default_value)
 {
   return gnc_option_db_lookup_multichoice_option(global_options, section,
                                                  name, default_value);
@@ -281,6 +321,48 @@ gnc_lookup_list_option(const char *section, const char *name,
 }
 
 
+/********************************************************************\
+ * gnc_lookup_currency_option                                       *
+ *   looks up a currency option.                                    *
+ *                                                                  *
+ * Args: section   - section name of option                         *
+ *       name      - name of option                                 *
+ *       default_value - default value to return if problem         *
+ * Return: currency object or NULL                                  *
+\********************************************************************/
+gnc_commodity *
+gnc_lookup_currency_option(const char *section,
+                           const char *name,
+                           gnc_commodity *default_value)
+{
+  return gnc_option_db_lookup_currency_option(global_options, section, name,
+                                              default_value);
+}
+
+
+/********************************************************************\
+ * gnc_default_currency                                             *
+ *   Return the default currency set by the user.                   *
+ *                                                                  *
+ * Args: section   - section name of option                         *
+ *       name      - name of option                                 *
+ *       default_value - default value to return if problem         *
+ * Return: currency object or NULL                                  *
+\********************************************************************/
+gnc_commodity *
+gnc_default_currency (void)
+{
+  gnc_commodity *currency;
+
+  currency = gnc_lookup_currency_option ("International",
+                                         "Default Currency", NULL);
+  if (currency)
+    return currency;
+
+  return gnc_locale_default_currency ();
+}
+
+
 /********************************************************************\
  * gnc_set_option_default                                           *
  *   set the option to its default value                            *
@@ -296,6 +378,23 @@ gnc_set_option_default(const char *section, const char *name)
 }
 
 
+/********************************************************************\
+ * gnc_set_option                                                   *
+ *   sets the option to the given value. If successful              *
+ *   returns TRUE, otherwise FALSE.                                 *
+ *                                                                  *
+ * Args: section   - section name of option                         *
+ *       name      - name of option                                 *
+ *       value     - value to set to                                *
+ * Return: success indicator                                        *
+\********************************************************************/
+gboolean
+gnc_set_option(const char *section, const char *name, SCM value)
+{
+  return gnc_option_db_set_option(global_options, section, name, value);
+}
+
+
 /********************************************************************\
  * gnc_set_number_option                                            *
  *   sets the number option to the given value. If successful       *
@@ -312,11 +411,27 @@ gnc_set_number_option(const char *section, const char *name, gdouble value)
   return gnc_option_db_set_number_option(global_options, section, name, value);
 }
 
+/********************************************************************\
+ * gnc_set_boolean_option                                           *
+ *   sets the boolean option to the given value. If successful      *
+ *   returns TRUE, otherwise FALSE.                                 *
+ *                                                                  *
+ * Args: section   - section name of option                         *
+ *       name      - name of option                                 *
+ *       value     - value to set to                                *
+ * Return: success indicator                                        *
+\********************************************************************/
+gboolean
+gnc_set_boolean_option(const char *section, const char *name, gboolean value)
+{
+  return gnc_option_db_set_boolean_option(global_options, section, name, value);
+}
+
 
 /********************************************************************\
  * _gnc_option_refresh_ui                                           *
- *   sets the GUI representation of an option with its current      *
- *   current guile value. this is intended for use by guile only    *
+ *   sets the GUI representation of an option with its              *
+ *   current guile value.  Designed to be called from guile         *
  *                                                                  *
  * Args: option - SCM handle to option                              *
  * Return: nothing                                                  *
@@ -330,6 +445,44 @@ _gnc_option_refresh_ui(SCM guile_option)
   gnc_option_set_ui_value(option, FALSE);
 }
 
+/********************************************************************\
+ *  gnc_option_refresh_ui_by_name                                   *
+ *   sets the GUI representation of an option with its current      *
+ *   current guile value.  Designed to be called from GUI           *
+ *                                                                  *
+ * Args: section_name: name of option's section                     *
+ *       name        : name of option                               *
+ * Return: nothing                                                  *
+\********************************************************************/
+void 
+gnc_option_refresh_ui_by_name(const char *section_name, const char *name)
+{
+  GNCOption *option;
+  option = gnc_option_db_get_option_by_name(global_options, section_name,
+					    name);
+  gnc_option_set_ui_value(option, FALSE);
+}
+
+/********************************************************************\
+ *  gnc_set_option_selectable_by_name                               *
+ *   sets the the sensitivity of a global option widget             *
+ *                                                                  *
+ * Args: section_name: name of option's section                     *
+ *       name        : name of option                               *
+ * Return: nothing                                                  *
+\********************************************************************/
+void
+gnc_set_option_selectable_by_name(const char *section_name, 
+                                  const char *name,
+                                  gboolean selectable)
+{
+  GNCOption *option;
+
+  option = gnc_option_db_get_option_by_name(global_options, section_name,
+                                            name);
+  if (option)
+    gnc_set_option_selectable (option, selectable);
+}
 
 /********************************************************************\
  * gnc_get_global_options                                           *
@@ -340,7 +493,7 @@ _gnc_option_refresh_ui(SCM guile_option)
  * Return: global options database                                  *
 \********************************************************************/
 GNCOptionDB *
-gnc_get_global_options()
+gnc_get_global_options(void)
 {
   return global_options;
 }
