@@ -447,7 +447,7 @@ gnc_scm2bitfield (const char *typestr, SCM field_scm)
 {
   int field = 0;
 
-  if (!gh_list_p (field))
+  if (!gh_list_p (field_scm))
     return 0;
 
   while (!gh_null_p (field_scm))
@@ -724,10 +724,29 @@ gnc_scm2query_term_query (SCM query_term_scm)
 
         amount = gh_scm2double (scm);
 
-        DxaccQueryAddAmountMatch (q, amount, amt_sgn, how, QUERY_OR);
+        switch (pr_type)
+        {
+          case PR_AMOUNT:
+            DxaccQueryAddAmountMatch (q, amount, amt_sgn, how, QUERY_OR);
+            ok = TRUE;
+            break;
+
+          case PR_PRICE:
+            DxaccQueryAddSharePriceMatch (q, amount, how, QUERY_OR);
+            ok = TRUE;
+            break;
+
+          case PR_SHRS:
+            DxaccQueryAddSharesMatch (q, amount, how, QUERY_OR);
+            ok = TRUE;
+            break;
+
+          default:
+            ok = FALSE;
+            break;
+        }
       }
 
-      ok = TRUE;
       break;
 
     case PD_ACCOUNT:
@@ -985,6 +1004,8 @@ gnc_scm2query_or_terms (SCM or_terms)
   if (!gh_list_p (or_terms))
     return NULL;
 
+  q = xaccMallocQuery ();
+
   while (!gh_null_p (or_terms))
   {
     SCM and_terms;
