@@ -23,14 +23,17 @@
  *
  *  Based heavily (i.e., cut and pasted from) on the Gnumeric ItemCursor.
  *
- * Author:
- *     Heath Martin <martinh@pegasus.cc.ucf.edu>
+ * Authors:
+ *     Heath Martin   <martinh@pegasus.cc.ucf.edu>
+ *     Dave Peticolas <dave@krondo.com>
  */
 
-#include "gnucash-sheet.h"
-#include "gnucash-grid.h"
+#include "config.h"
+
 #include "gnucash-color.h"
 #include "gnucash-cursor.h"
+#include "gnucash-grid.h"
+#include "gnucash-sheet.h"
 #include "gnucash-style.h"
 
 static GnomeCanvasItem *gnucash_cursor_parent_class;
@@ -44,26 +47,65 @@ enum {
 
 
 static void
-gnucash_cursor_get_pixel_coords (GnucashCursor *cursor, gint *x, gint *y,
+gnucash_cursor_get_pixel_coords (GnucashCursor *cursor,
+                                 gint *x, gint *y,
 				 gint *w, gint *h)
 {
         GnucashSheet *sheet = cursor->sheet;
-        GnucashItemCursor *item_cursor =
-		GNUCASH_ITEM_CURSOR(cursor->cursor[GNUCASH_CURSOR_BLOCK]);
+        GnucashItemCursor *item_cursor;
+        VirtualCellLocation vcell_loc;
+        CellDimensions *cd;
+        VirtualCell *vcell;
+        SheetBlock *block;
+        gint col;
 
-        gnome_canvas_get_scroll_offsets (GNOME_CANVAS(cursor->sheet), NULL, y);
+        item_cursor =
+                GNUCASH_ITEM_CURSOR(cursor->cursor[GNUCASH_CURSOR_BLOCK]);
 
-        *y += gnucash_sheet_row_get_distance (sheet, sheet->top_block,
-					      item_cursor->row)
-                + sheet->top_block_offset;
-        *x = gnucash_sheet_col_get_distance (sheet, item_cursor->row, sheet->left_block,
-					      item_cursor->col)
-                + sheet->left_block_offset;
+        vcell_loc.virt_row = item_cursor->row;
+        vcell_loc.virt_col = item_cursor->col;
 
-        *h = gnucash_sheet_row_get_distance (sheet, item_cursor->row,
-					     item_cursor->row + 1);
-        *w = gnucash_sheet_col_get_distance (sheet, item_cursor->row, item_cursor->col,
-					     item_cursor->col + 1);
+        block = gnucash_sheet_get_block (sheet, vcell_loc);
+        if (!block)
+                return;
+
+        vcell = gnc_table_get_virtual_cell (sheet->table, vcell_loc);
+        if (!vcell)
+                return;
+
+        for (col = 0; col < vcell->cellblock->num_cols; col++)
+        {
+                CellBlockCell *cb_cell;
+
+                cb_cell = gnc_cellblock_get_cell (vcell->cellblock, 0, col);
+                if (cb_cell->cell_type != NO_CELL)
+                        break;
+        }
+
+        *y = block->origin_y;
+
+        cd = gnucash_style_get_cell_dimensions (block->style, 0, col);
+        if (cd)
+                *x = cd->origin_x;
+        else
+                *x = block->origin_x;
+
+        for (col = vcell->cellblock->num_cols - 1; col >= 0; col--)
+        {
+                CellBlockCell *cb_cell;
+
+                cb_cell = gnc_cellblock_get_cell (vcell->cellblock, 0, col);
+                if (cb_cell->cell_type != NO_CELL)
+                        break;
+        }
+
+        *h = block->style->dimensions->height;
+
+        cd = gnucash_style_get_cell_dimensions (block->style, 0, col);
+        if (cd)
+                *w = cd->origin_x + cd->pixel_width - *x;
+        else
+                *w = block->style->dimensions->width - *x;
 }
 
 
@@ -93,53 +135,19 @@ gnucash_cursor_set_style (GnucashCursor  *cursor, SheetBlockStyle *style)
 
 
 void
-gnucash_cursor_get_phys (GnucashCursor *cursor, int *phys_row, int *phys_col)
+gnucash_cursor_get_virt (GnucashCursor *cursor, VirtualLocation *virt_loc)
 {
-        Table *table;
-        int virt_row, virt_col;
-        int cell_row, cell_col;
-
         g_return_if_fail (cursor != NULL);
         g_return_if_fail (GNUCASH_IS_CURSOR (cursor));
 
-        table = cursor->sheet->table;
-
-        virt_row =
+        virt_loc->vcell_loc.virt_row =
 		GNUCASH_ITEM_CURSOR(cursor->cursor[GNUCASH_CURSOR_BLOCK])->row;
-        virt_col =
+        virt_loc->vcell_loc.virt_col =
 		GNUCASH_ITEM_CURSOR(cursor->cursor[GNUCASH_CURSOR_BLOCK])->col;
 
-        cell_row =
+        virt_loc->phys_row_offset =
 		GNUCASH_ITEM_CURSOR(cursor->cursor[GNUCASH_CURSOR_CELL])->row;
-        cell_col =
-		GNUCASH_ITEM_CURSOR(cursor->cursor[GNUCASH_CURSOR_CELL])->col;
-
-        *phys_row =
-		table->rev_locators[virt_row][virt_col]->phys_row + cell_row;
-        *phys_col =
-		table->rev_locators[virt_row][virt_col]->phys_col + cell_col;
-}
-
-
-void
-gnucash_cursor_get_virt (GnucashCursor *cursor, int *virt_row, int *virt_col,
-                         int *cell_row, int *cell_col)
-{
-        Table *table;
-        
-        g_return_if_fail (cursor != NULL);
-        g_return_if_fail (GNUCASH_IS_CURSOR (cursor));
-
-        table = cursor->sheet->table;
-
-        *virt_row =
-		GNUCASH_ITEM_CURSOR(cursor->cursor[GNUCASH_CURSOR_BLOCK])->row;
-        *virt_col =
-		GNUCASH_ITEM_CURSOR(cursor->cursor[GNUCASH_CURSOR_BLOCK])->col;
-
-        *cell_row =
-		GNUCASH_ITEM_CURSOR(cursor->cursor[GNUCASH_CURSOR_CELL])->row;
-        *cell_col =
+        virt_loc->phys_col_offset =
 		GNUCASH_ITEM_CURSOR(cursor->cursor[GNUCASH_CURSOR_CELL])->col;
 }
 
@@ -148,19 +156,17 @@ void
 gnucash_cursor_configure (GnucashCursor *cursor)
 {
         GnomeCanvasItem *item;
-        GnucashItemCursor *item_cursor;
-        GnomeCanvasGroup *group;
+        GnucashItemCursor *block_cursor;
+        GnucashItemCursor *cell_cursor;
         GnomeCanvas *canvas;
         gint x, y, w, h;
         double wx, wy;
-                
+
         g_return_if_fail (cursor != NULL);
         g_return_if_fail (GNUCASH_IS_CURSOR (cursor));
 
         canvas = GNOME_CANVAS(GNOME_CANVAS_ITEM(cursor)->canvas);
 
-        group = GNOME_CANVAS_GROUP(cursor);
-
         item = GNOME_CANVAS_ITEM (cursor);
 
         gnucash_cursor_get_pixel_coords (cursor, &x, &y, &w, &h);
@@ -168,50 +174,50 @@ gnucash_cursor_configure (GnucashCursor *cursor)
 			       "GnomeCanvasGroup::x", (double)x,
 			       "GnomeCanvasGroup::y", (double)y,
 			       NULL);
-        
+
+        cursor->w = w;
+        cursor->h = h + 1;
+
         item->x1 = cursor->x = x;
         item->y1 = cursor->y = y;
-        cursor->w = w;
-        cursor->h = h;
-
         item->x2 = x + w;
-        item->y2 = y + h;
+        item->y2 = y + h + 1;
 
         item = cursor->cursor[GNUCASH_CURSOR_BLOCK];
-        item_cursor = GNUCASH_ITEM_CURSOR (item);
+        block_cursor = GNUCASH_ITEM_CURSOR (item);
 
         wx = 0;
         wy = 0;
 
         gnome_canvas_item_i2w (item, &wx, &wy);
-        gnome_canvas_w2c (canvas, wx, wy, &item_cursor->x, &item_cursor->y);
-        item_cursor->w = w;
-        item_cursor->h = h;
+        gnome_canvas_w2c (canvas, wx, wy, &block_cursor->x, &block_cursor->y);
+        block_cursor->w = w;
+        block_cursor->h = h + 1;
 
-        item->x1 = item_cursor->x;
-        item->y1 = item_cursor->y;
-        item->x2 = item_cursor->x + w;
-        item->y2 = item_cursor->y + h;
+        item->x1 = block_cursor->x;
+        item->y1 = block_cursor->y;
+        item->x2 = block_cursor->x + w;
+        item->y2 = block_cursor->y + h + 1;
 
         item = cursor->cursor[GNUCASH_CURSOR_CELL];
-        item_cursor = GNUCASH_ITEM_CURSOR(item);
-        
+        cell_cursor = GNUCASH_ITEM_CURSOR(item);
+
         gnucash_sheet_style_get_cell_pixel_rel_coords (cursor->style,
-						       item_cursor->row,
-						       item_cursor->col,
+						       cell_cursor->row,
+						       cell_cursor->col,
 						       &x, &y, &w, &h);
-        wx = x;
+        wx = x - block_cursor->x;
         wy = y;
 
         gnome_canvas_item_i2w (item, &wx, &wy);
-        gnome_canvas_w2c (canvas, wx, wy, &item_cursor->x, &item_cursor->y);
-        item_cursor->w = w;
-        item_cursor->h = h;
-        
-        item->x1 =  item_cursor->x;
-        item->y1 =  item_cursor->y;
-        item->x2 =  item_cursor->x+ w;
-        item->y2 =  item_cursor->y + h;
+        gnome_canvas_w2c (canvas, wx, wy, &cell_cursor->x, &cell_cursor->y);
+        cell_cursor->w = w;
+        cell_cursor->h = h;
+
+        item->x1 = cell_cursor->x;
+        item->y1 = cell_cursor->y;
+        item->x2 = cell_cursor->x + w;
+        item->y2 = cell_cursor->y + h;
 }
 
 
@@ -230,17 +236,17 @@ gnucash_item_cursor_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 			dw = item_cursor->w;
 			dh = item_cursor->h;
 
-			/* draw the double rectangle around the entire active 
+			/* draw the rectangle around the entire active
 			   virtual row */
 			gdk_gc_set_line_attributes (cursor->gc, 1,
 						    GDK_LINE_SOLID, -1, -1);
+
 			gdk_gc_set_foreground (cursor->gc, &gn_black);
-			gdk_gc_set_background (cursor->gc, &gn_white);
-                
+
 			gdk_draw_rectangle (drawable, cursor->gc, FALSE,
-					    dx+2, dy+2, dw-4, dh-4);
-			gdk_draw_rectangle (drawable, cursor->gc, FALSE,
-					    dx, dy, dw, dh);
+					    dx, dy, dw, dh - 1);
+                        gdk_draw_line (drawable, cursor->gc,
+                                       dx, dy + dh, dx + dw, dy + dh);
 
 			break;
 
@@ -250,17 +256,11 @@ gnucash_item_cursor_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 			dw = item_cursor->w;
 			dh = item_cursor->h;
 
-
 			gdk_gc_set_line_attributes (cursor->gc, 1,
 						    GDK_LINE_SOLID, -1, -1);
 
-			gdk_gc_set_foreground (cursor->gc, &gn_light_gray);
-			gdk_draw_rectangle (drawable, cursor->gc, FALSE,
-					    dx+1, dy+1, dw-2, dh-2);
+			gdk_gc_set_foreground (cursor->gc, &gn_black);
 
-			gdk_gc_set_foreground (cursor->gc, &gn_black);      
-			gdk_draw_rectangle (drawable, cursor->gc, FALSE,
-					    dx+2, dy+2, dw-4, dh-4);
 			gdk_draw_rectangle (drawable, cursor->gc, FALSE,
 					    dx, dy, dw, dh);
         }
@@ -268,8 +268,7 @@ gnucash_item_cursor_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 
 
 static void
-gnucash_cursor_set_block (GnucashCursor *cursor, gint block_row,
-			  gint block_col)
+gnucash_cursor_set_block (GnucashCursor *cursor, VirtualCellLocation vcell_loc)
 {
         GnucashSheet *sheet;
         GnucashItemCursor *item_cursor;
@@ -277,18 +276,20 @@ gnucash_cursor_set_block (GnucashCursor *cursor, gint block_row,
         g_return_if_fail (cursor != NULL);
         g_return_if_fail (GNUCASH_IS_CURSOR (cursor));
 
-        sheet  = cursor->sheet;
+        sheet = cursor->sheet;
         item_cursor =
 		GNUCASH_ITEM_CURSOR(cursor->cursor[GNUCASH_CURSOR_BLOCK]);
 
-        if (block_row < 0 || block_row >= sheet->num_virt_rows
-            || block_col < 0 || block_col >= sheet->num_virt_cols)
+        if (vcell_loc.virt_row < 0 ||
+            vcell_loc.virt_row >= sheet->num_virt_rows ||
+            vcell_loc.virt_col < 0 ||
+            vcell_loc.virt_col >= sheet->num_virt_cols)
                 return;
 
-        cursor->style = gnucash_sheet_get_style (sheet, block_row, block_col);
+        cursor->style = gnucash_sheet_get_style (sheet, vcell_loc);
 
-        item_cursor->row = block_row;
-        item_cursor->col = block_col;
+        item_cursor->row = vcell_loc.virt_row;
+        item_cursor->col = vcell_loc.virt_col;
 }
 
 
@@ -302,24 +303,21 @@ gnucash_cursor_set_cell (GnucashCursor *cursor, gint cell_row, gint cell_col)
         g_return_if_fail (cursor != NULL);
         g_return_if_fail (GNUCASH_IS_CURSOR (cursor));
 
-        sheet  = cursor->sheet;
+        sheet = cursor->sheet;
         item_cursor = GNUCASH_ITEM_CURSOR(cursor->cursor[GNUCASH_CURSOR_CELL]);
         style = cursor->style;
 
-        if (cell_row < 0 || cell_row >= style->nrows
-            || cell_col < 0 || cell_col >= style->ncols)
+        if (cell_row < 0 || cell_row >= style->nrows ||
+            cell_col < 0 || cell_col >= style->ncols)
                 return;
 
         item_cursor->row = cell_row;
         item_cursor->col = cell_col;
 }
 
-                   
-
 
 void
-gnucash_cursor_set (GnucashCursor *cursor, gint block_row, gint block_col,
-                    gint cell_row, gint cell_col)
+gnucash_cursor_set (GnucashCursor *cursor, VirtualLocation virt_loc)
 {
         GnucashSheet *sheet;
         
@@ -330,8 +328,10 @@ gnucash_cursor_set (GnucashCursor *cursor, gint block_row, gint block_col,
 
         gnucash_cursor_request_redraw (cursor);
 
-        gnucash_cursor_set_block (cursor, block_row, block_col);
-        gnucash_cursor_set_cell (cursor, cell_row, cell_col);
+        gnucash_cursor_set_block (cursor, virt_loc.vcell_loc);
+        gnucash_cursor_set_cell (cursor,
+                                 virt_loc.phys_row_offset,
+                                 virt_loc.phys_col_offset);
 
         gnucash_cursor_configure (cursor);
 
@@ -554,21 +554,20 @@ gnucash_cursor_new (GnomeCanvasGroup *parent)
 
         g_return_val_if_fail (parent != NULL, NULL);
         g_return_val_if_fail (GNOME_IS_CANVAS_GROUP(parent), NULL);
-        
 
         item = gnome_canvas_item_new (parent,
                                       gnucash_cursor_get_type(),
                                       NULL);
 
         cursor = GNUCASH_CURSOR(item);
-        
+
         cursor_item = gnome_canvas_item_new (GNOME_CANVAS_GROUP(item),
 					     gnucash_item_cursor_get_type(),
 					     NULL);
 
         item_cursor = GNUCASH_ITEM_CURSOR (cursor_item);
         item_cursor->type = GNUCASH_CURSOR_CELL;
-        
+
         cursor->cursor[GNUCASH_CURSOR_CELL] = cursor_item;
 
         cursor_item = gnome_canvas_item_new (GNOME_CANVAS_GROUP(item),
@@ -577,7 +576,7 @@ gnucash_cursor_new (GnomeCanvasGroup *parent)
 
         item_cursor = GNUCASH_ITEM_CURSOR (cursor_item);
         item_cursor->type = GNUCASH_CURSOR_BLOCK;
-        
+
         cursor->cursor[GNUCASH_CURSOR_BLOCK] = cursor_item;
 
         return item;
