@@ -21,10 +21,13 @@
 /*
  * The Gnucash Header Canvas
  *
- * Author:
+ * Authors:
  *     Heath Martin <martinh@pegasus.cc.ucf.edu>
+ *     Dave Peticolas <dave@krondo.com>
  */
 
+#include "config.h"
+
 #include "gnucash-sheet.h"
 #include "gnucash-color.h"
 #include "gnucash-style.h"
@@ -53,8 +56,8 @@ gnucash_header_update (GnomeCanvasItem *item, double *affine,
 
         item->x1 = 0;
         item->y1 = 0;
-        item->x2 = INT_MAX/2 -1;
-        item->y2 = INT_MAX/2 -1;
+        item->x2 = (INT_MAX / 2) - 1;
+        item->y2 = (INT_MAX / 2) - 1;
 }
 
 
@@ -64,97 +67,143 @@ gnucash_header_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 {
         GnucashHeader *header = GNUCASH_HEADER(item);
         SheetBlockStyle *style = header->style;
-        SheetBlockStyle *header_style;
-        int i, j;
+        Table *table = header->sheet->table;
+        VirtualLocation virt_loc;
+        VirtualCell *vcell;
+        CellDimensions *cd;
+        GdkColor *bg_color;
         int xpaint, ypaint;
-        int w = 0, h = 0;
-        gchar *text;
+        const char *text;
+        CellBlock *cb;
         GdkFont *font;
+        guint32 argb;
+        int i, j;
+        int w, h;
 
-        header_style = header->sheet->cursor_style[GNUCASH_CURSOR_HEADER];
+        virt_loc.vcell_loc.virt_row = 0;
+        virt_loc.vcell_loc.virt_col = 0;
+        virt_loc.phys_row_offset = 0;
+        virt_loc.phys_col_offset = 0;
 
-        /* Assume all cells have the same color */
-        gdk_gc_set_foreground(header->gc,
-                              header_style->inactive_bg_color[0][0]);
-        gdk_draw_rectangle(drawable, header->gc, TRUE, 0, 0, width, height);
+        argb = gnc_table_get_bg_color (table, virt_loc, NULL);
+
+        bg_color = gnucash_color_argb_to_gdk (argb);
+
+        h = style->dimensions->height;
+        if (header->style->nrows == 1 &&
+            header->sheet->table->num_header_phys_rows == 2)
+                h *= 2;
+
+        gdk_gc_set_foreground (header->gc, bg_color);
+
+        gdk_draw_rectangle (drawable, header->gc, TRUE, 0, 0,
+                            style->dimensions->width, h);
 
         gdk_gc_set_line_attributes (header->gc, 1, GDK_LINE_SOLID, -1, -1);
         gdk_gc_set_foreground (header->gc, &gn_black);
+
         gdk_draw_rectangle (drawable, header->gc, FALSE, -x, -y,
-                            style->dimensions->width,
-                            style->dimensions->height);
-        gdk_draw_line (drawable, header->gc, -x,
-                       style->dimensions->height - 1,
-                       style->dimensions->width - 1,
-                       style->dimensions->height - 1);
+                            style->dimensions->width, h);
+
+        gdk_draw_line (drawable, header->gc, 0, h + 1,
+                       style->dimensions->width, h + 1);
 
         gdk_gc_set_line_attributes (header->gc, 1, GDK_LINE_SOLID, -1, -1);
         gdk_gc_set_background (header->gc, &gn_white);
         gdk_gc_set_foreground (header->gc, &gn_black);
-        font = style->header_font;
+        font = gnucash_register_font;
+
+        vcell = gnc_table_get_virtual_cell
+                (table, table->current_cursor_loc.vcell_loc);
+        cb = vcell ? vcell->cellblock : NULL;
 
         ypaint = -y;
+        h = 0;
 
-        for (i = 0; i < style->nrows; i++) {
+        for (i = 0; i < style->nrows; i++)
+        {
                 xpaint = -x;
+                virt_loc.phys_row_offset = i;
 
                 /* TODO: This routine is duplicated in several places.
                    Can we abstract at least the cell drawing routine?
                    That way we'll be sure everything is drawn
                    consistently, and cut down on maintenance issues. */
-                for (j = 0; j < style->ncols; j++) {
+
+                for (j = 0; j < style->ncols; j++)
+                {
+                        gint x_offset, y_offset;
+                        GdkRectangle rect;
+                        CellBlockCell *cb_cell;
+
+                        virt_loc.phys_col_offset = j;
+
+                        cd = gnucash_style_get_cell_dimensions (style, i, j);
+
                         if (header->in_resize && (j == header->resize_col))
                                 w = header->resize_col_width;
                         else
-                                w = style->dimensions->pixel_widths[i][j];
-                        
-                        h = style->dimensions->pixel_heights[i][j];
-                        
+                                w = cd->pixel_width;
+
+                        cb_cell = gnc_cellblock_get_cell (cb, i, j);
+                        if (!cb_cell || cb_cell->cell_type < 0)
+                        {
+                                xpaint += w;
+                                continue;
+                        }
+
+                        h = cd->pixel_height;
+
                         gdk_draw_rectangle (drawable, header->gc, FALSE,
                                             xpaint, ypaint, w, h);
-                        
-                        text = style->labels[i][j];
-                        
-                        if (text) {
-                                gint x_offset, y_offset;
-                                GdkRectangle rect;
 
-                                y_offset = h - MAX(CELL_VPADDING,
-                                                   font->descent + 4);
+                        virt_loc.vcell_loc =
+                                table->current_cursor_loc.vcell_loc;
+                        text = gnc_table_get_label (table, virt_loc);
+                        if (!text)
+                                text = "";
 
-                                switch (style->alignments[i][j]) {
-                                default:
-                                case GTK_JUSTIFY_LEFT:
-                                case GTK_JUSTIFY_FILL:
-                                case GTK_JUSTIFY_CENTER:
+                        y_offset = ((h / 2) +
+                                    (((font->ascent + font->descent) / 2) -
+                                     font->descent));
+                        y_offset++;
+
+                        switch (gnc_table_get_align (table, virt_loc)) {
+                        default:
+                        case CELL_ALIGN_LEFT:
+                                x_offset = CELL_HPADDING;
+                                break;
+                        case CELL_ALIGN_RIGHT:
+                                x_offset = w - CELL_HPADDING;
+                                x_offset -= gdk_string_measure(font, text);
+                                break;
+                        case CELL_ALIGN_CENTER:
+                                if (w < gdk_string_measure (font, text))
                                         x_offset = CELL_HPADDING;
-                                        break;
-                                case GTK_JUSTIFY_RIGHT:
-                                        x_offset = w - CELL_HPADDING
-                                                - gdk_string_measure(font,
-                                                                     text);
-                                        break;
+                                else
+                                {
+                                        x_offset = w / 2;
+                                        x_offset -= gdk_string_measure (font, text) / 2;
                                 }
-
-                                rect.x = xpaint + CELL_HPADDING;
-                                rect.y = ypaint + CELL_VPADDING;
-                                rect.width = MAX(0, w - 2*CELL_HPADDING);
-                                rect.height = h;
-                                
-                                gdk_gc_set_clip_rectangle (header->gc, &rect);
-                                
-                                gdk_draw_string (drawable,
-                                                 font,
-                                                 header->gc,
-                                                 xpaint + x_offset,
-                                                 ypaint + y_offset,
-                                                 text);
-                                gdk_gc_set_clip_rectangle (header->gc, NULL);
-                                
+                                break;
                         }
-                        
+
+                        rect.x = xpaint + CELL_HPADDING;
+                        rect.y = ypaint + 1;
+                        rect.width = MAX (0, w - (2 * CELL_HPADDING));
+                        rect.height = h - 2;
+
+                        gdk_gc_set_clip_rectangle (header->gc, &rect);
+
+                        gdk_draw_string (drawable, font, header->gc,
+                                         xpaint + x_offset,
+                                         ypaint + y_offset, text);
+
+                        gdk_gc_set_clip_rectangle (header->gc, NULL);
+
                         xpaint += w;
                 }
+
                 ypaint += h;
         }
 }
@@ -165,7 +214,12 @@ gnucash_header_request_redraw (GnucashHeader *header)
 {
         GnomeCanvas *canvas = GNOME_CANVAS_ITEM(header)->canvas;
 
-        gnome_canvas_request_redraw (canvas, 0, 0, INT_MAX/2 -1, INT_MAX/2 -1);
+        if (header->style == NULL)
+                return;
+
+        gnome_canvas_request_redraw (canvas, 0, 0,
+                                     header->style->dimensions->width + 1,
+                                     header->style->dimensions->height + 1);
 }
 
 
@@ -175,13 +229,12 @@ gnucash_header_realize (GnomeCanvasItem *item)
         GnucashHeader *header = GNUCASH_HEADER (item);
         GdkWindow *window;
 
-        	
         if (GNOME_CANVAS_ITEM_CLASS (gnucash_header_parent_class)->realize)
                 (*GNOME_CANVAS_ITEM_CLASS
 		 (gnucash_header_parent_class)->realize)(item);
 
         window = GTK_WIDGET (item->canvas)->window;
-        
+
         header->gc = gdk_gc_new (window);
 }
 
@@ -231,13 +284,13 @@ gnucash_header_reconfigure (GnucashHeader *header)
 
         g_return_if_fail (header != NULL);
         g_return_if_fail (GNUCASH_IS_HEADER (header));
-        
+
         canvas = GNOME_CANVAS_ITEM(header)->canvas;
         widget = GTK_WIDGET (header->sheet);
         sheet = GNUCASH_SHEET(header->sheet);
         old_style = header->style;
 
-        header->style = header->sheet->cursor_style[header->type];
+        header->style = header->sheet->cursor_styles[header->type];
 
         if (header->style == NULL)
                 return;
@@ -249,23 +302,29 @@ gnucash_header_reconfigure (GnucashHeader *header)
 
         sheet->width = header->style->dimensions->width;
 
-        w = MAX (widget->allocation.width, header->style->dimensions->width);
+        w = header->style->dimensions->width;
         h = header->style->dimensions->height;
+        if (header->style->nrows == 1 &&
+            header->sheet->table->num_header_phys_rows == 2)
+                h *= 2;
+        h += 2;
 
-        if (header->height != h || header->width != w ||
-            header->style != old_style) {
+        if (header->height != h ||
+            header->width != w  ||
+            header->style != old_style)
+        {
                 header->height = h;
                 header->width = w;
 
-                gnome_canvas_set_scroll_region(GNOME_CANVAS(canvas),
-					       0, 0, w, h);
+                gnome_canvas_set_scroll_region (GNOME_CANVAS(canvas),
+                                                0, 0, w, h);
+
                 gtk_widget_set_usize (GTK_WIDGET(canvas), -1, h);
 
                 gnucash_header_request_redraw (header);
         }
 }
 
-
 static double
 gnucash_header_point (GnomeCanvasItem *item,
                       double x, double y, int cx, int cy,
@@ -285,11 +344,13 @@ pointer_on_resize_line (GnucashHeader *header, int x, int y, int *col)
 {
         SheetBlockStyle *style = header->style;
         gboolean on_the_line = FALSE;
-        int pixels = 0; /* = style->dimensions->pixel_widths[header->row][0];*/
+        CellDimensions *cd;
+        int pixels = 0;
         int j;
 
         for (j = 0; j < style->ncols; j++) {
-                pixels += style->dimensions->pixel_widths[header->row][j];
+                cd = gnucash_style_get_cell_dimensions (style, header->row, j);
+                pixels += cd->pixel_width;
                 if (x >= pixels - 1 && x <= pixels + 1)
                         on_the_line = TRUE;
                 if (x <= pixels + 1)
@@ -306,15 +367,17 @@ pointer_on_resize_line (GnucashHeader *header, int x, int y, int *col)
 static int
 find_resize_col (GnucashHeader *header, int col)
 {
-        int start = col;
         SheetBlockStyle *style = header->style;
+        CellDimensions *cd;
+        int start = col;
 
         if (col < 0 || col >= style->ncols)
                 return -1;
 
         /* skip to the right over zero-width columns */
-        while (col+1 < style->ncols &&
-               style->dimensions->pixel_widths[0][col+1] == 0)
+        while ((col+1 < style->ncols) &&
+               (cd = gnucash_style_get_cell_dimensions (style, 0, col + 1)) &&
+               (cd->pixel_width == 0))
                 col++;
 
         /* now go back left till we have a resizable column */
@@ -330,26 +393,33 @@ find_resize_col (GnucashHeader *header, int col)
 }
 
 static void
-gnucash_header_auto_resize_column (GnucashHeader *header, gint col)
+gnucash_header_resize_column (GnucashHeader *header, gint col, gint width)
 {
         GnucashSheet *sheet = header->sheet;
 
-        int width = gnucash_sheet_col_max_width (sheet, 0, col);
-
-        gnucash_sheet_style_set_col_width (sheet, header->style,
-                                           col, width, FALSE);
-
-        gtk_window_set_default_size(GTK_WINDOW(sheet->window), 0, 0); 
+        gnucash_sheet_set_col_width (sheet, col, width);
 
         gnucash_cursor_configure (GNUCASH_CURSOR(sheet->cursor));
         item_edit_configure (ITEM_EDIT(sheet->item_editor));
 
+        gnucash_header_reconfigure (header);
+        gnucash_sheet_set_scroll_region (sheet);
         gnucash_sheet_update_adjustments (sheet);
 
         gnucash_header_request_redraw (header);
         gnucash_sheet_redraw_all (sheet);
 }
 
+static void
+gnucash_header_auto_resize_column (GnucashHeader *header, gint col)
+{
+        int width;
+
+        width = gnucash_sheet_col_max_width (header->sheet, 0, col);
+
+        gnucash_header_resize_column (header, col, width);
+}
+
 static gint
 gnucash_header_event (GnomeCanvasItem *item, GdkEvent *event)
 {
@@ -413,9 +483,15 @@ gnucash_header_event (GnomeCanvasItem *item, GdkEvent *event)
                         col = -1;
 
                 if (col > -1) {
+                        CellDimensions *cd;
+
+                        cd = gnucash_style_get_cell_dimensions (header->style,
+                                                                header->row,
+                                                                col);
+
                         header->in_resize = TRUE;
                         header->resize_col = col;
-                        header->resize_col_width = header->style->dimensions->pixel_widths[header->row][col];
+                        header->resize_col_width = cd->pixel_width;
                         header->resize_x = x;
                 }
 
@@ -423,8 +499,6 @@ gnucash_header_event (GnomeCanvasItem *item, GdkEvent *event)
         }
         case GDK_BUTTON_RELEASE: 
         {
-                GnucashSheet *sheet = header->sheet;
-
                 if (event->button.button != 1)
                         break;
 
@@ -433,21 +507,14 @@ gnucash_header_event (GnomeCanvasItem *item, GdkEvent *event)
 
                 if (header->in_resize) {
                         if (header->needs_ungrab) {
-                                gnome_canvas_item_ungrab (item, event->button.time);
+                                gnome_canvas_item_ungrab (item,
+                                                          event->button.time);
                                 header->needs_ungrab = FALSE;
 
-                                gnucash_sheet_style_set_col_width (sheet, header->style,
-                                                                   header->resize_col, header->resize_col_width, FALSE);
-
-                                gtk_window_set_default_size(GTK_WINDOW(sheet->window), 0, 0);
-
-                                gnucash_cursor_configure (GNUCASH_CURSOR(sheet->cursor));
-                                item_edit_configure (ITEM_EDIT(sheet->item_editor));
-
-                                gnucash_sheet_update_adjustments (sheet);
-
-                                gnucash_header_request_redraw (header);
-                                gnucash_sheet_redraw_all (sheet);
+                                gnucash_header_resize_column
+                                        (header,
+                                         header->resize_col,
+                                         header->resize_col_width);
                         }
                         header->in_resize = FALSE;
                         header->resize_col = -1;
@@ -476,7 +543,8 @@ gnucash_header_event (GnomeCanvasItem *item, GdkEvent *event)
                         header->in_resize = FALSE;
                         header->resize_col = -1;
                         if (header->needs_ungrab) {
-                                gnome_canvas_item_ungrab (item, event->button.time);
+                                gnome_canvas_item_ungrab (item,
+                                                          event->button.time);
                                 header->needs_ungrab = FALSE;
                         }
 
@@ -513,7 +581,7 @@ gnucash_header_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
         case ARG_CURSOR_TYPE: 
         {
                 gint old_type = header->type;
-                
+
                 header->type = GTK_VALUE_INT (*arg);
                 needs_update = (old_type != header->type);
                 break;
@@ -598,7 +666,7 @@ static void
 gnucash_header_realized (GtkWidget *widget, gpointer data)
 {
        	gdk_window_set_back_pixmap (GTK_LAYOUT (widget)->bin_window,
-				    NULL, FALSE);
+                                    NULL, FALSE);
 }
 
 
@@ -621,12 +689,13 @@ gnucash_header_new (GnucashSheet *sheet)
                                       gnucash_header_get_type (),
                                       "GnucashHeader::sheet", sheet,
                                       "GnucashHeader::cursor_type",
-                                      GNUCASH_CURSOR_HEADER,
+                                      CURSOR_TYPE_HEADER,
                                       NULL);
 
         sheet->header_item = item;
 
         gtk_widget_show (canvas);
+
         return canvas;
 }
 
