@@ -1,4 +1,6 @@
 /********************************************************************\
+ * gnucash-item-list.c -- A scrollable list box                     *
+ *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
  * published by the Free Software Foundation; either version 2 of   *
@@ -24,8 +26,9 @@
 
 #include <gnome.h>
 
+#include "gnc-engine-util.h"
 #include "gnucash-item-list.h"
-#include "util.h"
+#include "gnucash-scrolled-window.h"
 
 
 /* Item list signals */
@@ -135,26 +138,13 @@ gnc_item_list_sort(GNCItemList *item_list)
 }
 
 
-void
+int
 gnc_item_list_autosize(GNCItemList *item_list)
 {
-        g_return_if_fail(item_list != NULL);
-	g_return_if_fail(IS_GNC_ITEM_LIST(item_list));
+        g_return_val_if_fail(item_list != NULL, 0);
+        g_return_val_if_fail(IS_GNC_ITEM_LIST(item_list), 0);
 
-        gtk_clist_columns_autosize(item_list->clist);
-}
-
-
-static void
-item_edit_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
-{
-	/*        GNCItemList *item_list = GNC_ITEM_LIST(object); */
-
-        switch (arg_id)
-	{
-		default:
-			break;
-        }
+        return gtk_clist_columns_autosize(item_list->clist);
 }
 
 
@@ -258,12 +248,10 @@ gnc_item_list_class_init(GNCItemListClass *item_list_class)
         GtkObjectClass  *object_class;
 
         gnc_item_list_parent_class =
-		gtk_type_class(gnome_canvas_widget_get_type());
+		gtk_type_class (gnome_canvas_widget_get_type ());
 
         object_class = GTK_OBJECT_CLASS(item_list_class);
 
-        object_class->set_arg = item_edit_set_arg;
-
 	gnc_item_list_signals[SELECT_ITEM] =
 		gtk_signal_new("select_item",
 			       GTK_RUN_LAST,
@@ -368,34 +356,40 @@ clist_select_row_cb(GtkCList *clist, gint row, gint column,
 GnomeCanvasItem *
 gnc_item_list_new(GnomeCanvasGroup *parent)
 {
-	GtkWidget *clist, *hbox, *scrollbar;
+        GtkWidget *frame;
+	GtkWidget *clist;
+        GtkWidget *scrollwin;
         GnomeCanvasItem *item;
         GNCItemList *item_list;
 
-	hbox = gtk_hbox_new(FALSE, 0);
+        frame = gtk_frame_new (NULL);
 
-	clist = gtk_clist_new(1);
-	gtk_box_pack_start(GTK_BOX(hbox), clist, TRUE, TRUE, 0);
+        scrollwin = gnc_scrolled_window_new ();
+        gtk_container_add (GTK_CONTAINER (frame), scrollwin);
+
+        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin),
+                                        GTK_POLICY_AUTOMATIC,
+                                        GTK_POLICY_AUTOMATIC);
+
+	clist = gtk_clist_new (1);
+        gtk_container_add (GTK_CONTAINER (scrollwin), clist);
         gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
 
-	scrollbar = gtk_vscrollbar_new(NULL);
-	gtk_box_pack_start(GTK_BOX(hbox), scrollbar, FALSE, FALSE, 0);
-	gtk_clist_set_vadjustment(GTK_CLIST(clist),
-				  gtk_range_get_adjustment
-				  (GTK_RANGE(scrollbar)));
+        gtk_widget_show_all (frame);
 
-        item = gnome_canvas_item_new(parent, gnc_item_list_get_type(),
-				     "widget", hbox,
-				     "size_pixels", TRUE,
-				     "x", -10000.0,
-                                     "y", -10000.0,
-				     NULL);
+        item = gnome_canvas_item_new (parent, gnc_item_list_get_type(),
+                                      "widget", frame,
+                                      "size_pixels", TRUE,
+                                      "x", -10000.0,
+                                      "y", -10000.0,
+                                      NULL);
 
-        item_list = GNC_ITEM_LIST(item);
+        item_list = GNC_ITEM_LIST (item);
 
-	item_list->clist = GTK_CLIST(clist);
+	item_list->clist = GTK_CLIST (clist);
+        item_list->frame = frame;
 
-	gtk_signal_connect_after(GTK_OBJECT(hbox), "button_press_event",
+	gtk_signal_connect_after(GTK_OBJECT(frame), "button_press_event",
 				 GTK_SIGNAL_FUNC(gnc_item_list_button_event),
 				 (gpointer) item_list);
 
