@@ -1,4 +1,6 @@
 /********************************************************************\
+ * gnucash-item-list.h -- A scrollable list box                     *
+ *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
  * published by the Free Software Foundation; either version 2 of   *
@@ -22,12 +24,6 @@
 #define GNUCASH_ITEM_LIST_H
 
 
-#include "gnucash-sheet.h"
-#include "gnucash-color.h"
-#include "gnucash-grid.h"
-#include "gnucash-cursor.h"
-
-
 #define GNC_ITEM_LIST(obj) (GTK_CHECK_CAST((obj), gnc_item_list_get_type (), GNCItemList))
 #define GNC_ITEM_LIST_CLASS(k) (GTK_CHECK_CLASS_CAST ((k), gnc_item_list_get_type (), GNCItemListClass))
 #define IS_GNC_ITEM_LIST(o) (GTK_CHECK_TYPE((o), gnc_item_list_get_type ()))
@@ -37,7 +33,8 @@ typedef struct
 {
         GnomeCanvasWidget canvas_widget;
 
-        GtkCList *clist; /* Contains the list items */
+        GtkCList *clist;  /* Contains the list items */
+        GtkWidget *frame; /* frame around everything */
 } GNCItemList;
 
 
@@ -45,17 +42,17 @@ GtkType gnc_item_list_get_type (void);
 
 GnomeCanvasItem *gnc_item_list_new (GnomeCanvasGroup *parent);
 
-void gnc_item_list_clear(GNCItemList *item_list);
+void gnc_item_list_clear (GNCItemList *item_list);
 
-void gnc_item_list_append(GNCItemList *item_list, char *string);
+void gnc_item_list_append (GNCItemList *item_list, char *string);
 
-void gnc_item_list_select(GNCItemList *item_list, const char *string);
+void gnc_item_list_select (GNCItemList *item_list, const char *string);
 
-void gnc_item_list_show_selected(GNCItemList *item_list);
+void gnc_item_list_show_selected (GNCItemList *item_list);
 
-void gnc_item_list_sort(GNCItemList *item_list);
+void gnc_item_list_sort (GNCItemList *item_list);
 
-void gnc_item_list_autosize(GNCItemList *item_list);
+int gnc_item_list_autosize (GNCItemList *item_list);
 
 
 typedef struct
