@@ -1,4 +1,6 @@
 /********************************************************************\
+ * numcell.c -- number handling cell incl. accelarator key support  *
+ *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
  * published by the Free Software Foundation; either version 2 of   *
@@ -34,113 +36,95 @@
 #include <string.h>
 
 #include "numcell.h"
-#include "util.h"
+#include "gnc-engine-util.h"
+
 
 /* This static indicates the debugging module that this .o belongs to. */
 /* static short module = MOD_REGISTER; */
 
+static void xaccInitNumCell (NumCell *cell);
+
 
 /* ================================================ */
 /* Parses the string value and returns true if it is a
  * number. In that case, *num is set to the value parsed. */
-static gncBoolean
+static gboolean
 parse_num(const char *string, long int *num)
 {
   long int number;
 
   if (string == NULL)
-    return GNC_F;
+    return FALSE;
 
   if (!gnc_strisnum(string))
-    return GNC_F;
+    return FALSE;
 
   number = strtol(string, NULL, 10);
 
   if ((number == LONG_MIN) || (number == LONG_MAX))
-    return GNC_F;
+    return FALSE;
 
   if (num != NULL)
     *num = number;
 
-  return GNC_T;
+  return TRUE;
 }
 
 /* ================================================ */
-static const char * 
-NumEnter (BasicCell *_cell,
-          const char * curr,
-          int *cursor_position,
-          int *start_selection,
-          int *end_selection)
-{
-  NumCell *cell = (NumCell *) _cell;
-
-  if (!cell->next_num_set)
-  {
-    long int number;
-
-    if (parse_num(curr, &number))
-      cell->next_num = number + 1;
-  }
-
-  return curr;
-}
-
-/* ================================================ */
-static const char * 
+static void
 NumMV (BasicCell *_cell, 
-       const char *oldval, 
-       const char *change, 
-       const char *newval,
+       const GdkWChar *change,
+       int change_len,
+       const GdkWChar *newval,
+       int new_val_len,
        int *cursor_position,
        int *start_selection,
        int *end_selection)
 {
   NumCell *cell = (NumCell *) _cell;
-  gncBoolean accel = GNC_F;
-  gncBoolean is_num;
+  gboolean accel = FALSE;
+  gboolean is_num;
   long int number = 0;
 
-  if ((change == NULL) || (change[0] == 0) || /* if we are deleting       */
-      (strlen(change) > 1))                   /* or entering > 1 char     */
+  if ((change == NULL) || (change_len == 0) || /* if we are deleting       */
+      (change_len > 1))                       /* or entering > 1 char     */
     /* then just accept the proposed change */
   {
-    if (cell->cell.value) free (cell->cell.value);
-    cell->cell.value = strdup (newval);
-    return newval;
+    xaccSetBasicCellWCValueInternal (&cell->cell, newval);
+    return;
   }
 
   /* otherwise, it may be an accelerator key. */
 
-  is_num = parse_num(oldval, &number);
+  is_num = parse_num (_cell->value, &number);
 
   if (is_num && (number < 0))
-    is_num = GNC_F;
+    is_num = FALSE;
 
   switch (change[0])
   {
     case '+':
     case '=':
       number++;
-      accel = GNC_T;
+      accel = TRUE;
       break;
 
     case '_':
     case '-':
       number--;
-      accel = GNC_T;
+      accel = TRUE;
       break;
 
     case '}':
     case ']':
       number += 10;
-      accel = GNC_T;
+      accel = TRUE;
       break;
 
     case '{':
     case '[':
       number -= 10;
-      accel = GNC_T;
+      accel = TRUE;
       break;
   }
 
@@ -148,8 +132,8 @@ NumMV (BasicCell *_cell,
     number = 0;
 
   /* If there is already a non-number there, don't accelerate. */
-  if (accel && !is_num && (safe_strcmp(oldval, "") != 0))
-    accel = GNC_F;
+  if (accel && !is_num && (safe_strcmp(_cell->value, "") != 0))
+    accel = FALSE;
 
   if (accel)
   {
@@ -158,41 +142,20 @@ NumMV (BasicCell *_cell,
     if (!is_num)
       number = cell->next_num;
 
-    strcpy(buff, "");
-    snprintf(buff, sizeof(buff), "%ld", number);
+    strcpy (buff, "");
+    snprintf (buff, sizeof(buff), "%ld", number);
 
     if (safe_strcmp(buff, "") == 0)
-      return NULL;
+      return;
 
-    if (cell->cell.value) free (cell->cell.value);
-    cell->cell.value = strdup (buff);
+    xaccSetBasicCellValueInternal (&cell->cell, buff);
 
     *cursor_position = -1;
 
-    return strdup(buff);
+    return;
   }
 
-  if (cell->cell.value) free (cell->cell.value);
-  cell->cell.value = strdup (newval);
-
-  return newval;
-}
-
-/* ================================================ */
-static const char * 
-NumLeave (BasicCell *_cell, const char * curr)
-{
-  NumCell *cell = (NumCell *) _cell;
-
-  if (!cell->next_num_set)
-  {
-    long int number;
-
-    if (parse_num(curr, &number))
-      cell->next_num = number + 1;
-  }
-
-  return NULL;
+  xaccSetBasicCellWCValueInternal (&cell->cell, newval);
 }
 
 /* ================================================ */
@@ -201,7 +164,8 @@ xaccMallocNumCell (void)
 {
   NumCell *cell;
 
-  cell = (NumCell *) malloc(sizeof(NumCell));
+  cell = g_new0 (NumCell, 1);
+
   xaccInitNumCell (cell);
 
   return cell;
@@ -228,41 +192,47 @@ setNumCellValue (BasicCell *_cell, const char *str)
       cell->next_num = number + 1;
   }
 
-  if (cell->cell.value) free (cell->cell.value);
-  cell->cell.value = strdup (str);
+  xaccSetBasicCellValueInternal (_cell, str);
 }
 
 /* ================================================ */
 void 
 xaccSetNumCellValue (NumCell *cell, const char *str)
 {
-  setNumCellValue(&cell->cell, str);
+  if (!cell)
+    return;
+
+  setNumCellValue (&cell->cell, str);
 }
 
 /* ================================================ */
-void
+gboolean
 xaccSetNumCellLastNum (NumCell *cell, const char *str)
 {
   long int number;
 
-  if (parse_num(str, &number))
+  if (!cell)
+    return FALSE;
+
+  if (parse_num (str, &number))
   {
     cell->next_num = number + 1;
-    cell->next_num_set = GNC_T;
+    cell->next_num_set = TRUE;
+    return TRUE;
   }
+
+  return FALSE;
 }
 
 /* ================================================ */
-void
+static void
 xaccInitNumCell (NumCell *cell)
 {
   xaccInitBasicCell (&(cell->cell));
 
   cell->next_num = 0;
-  cell->next_num_set = GNC_F;
+  cell->next_num_set = FALSE;
  
-  cell->cell.enter_cell = NumEnter;
   cell->cell.modify_verify = NumMV;
-  cell->cell.leave_cell = NumLeave;
   cell->cell.set_value = setNumCellValue;
 }
