@@ -169,10 +169,12 @@
      (- tree-depth (+ current-depth (if group-header-line? 1 0))))
     ;; the account balance
     (list (and my-balance
-	       (gnc:make-html-text
-		((if boldface? gnc:html-markup-b identity)
-		 ((if reverse-balance? gnc:monetary-neg identity)
-		  my-balance)))))
+	       (gnc:make-html-table-cell/markup 
+		"number-cell"
+		(gnc:make-html-text
+		 ((if boldface? gnc:html-markup-b identity)
+		  ((if reverse-balance? gnc:monetary-neg identity)
+		   my-balance))))))
     (gnc:html-make-empty-cells (- current-depth 
 				  (if group-header-line? 0 1))))))
 
@@ -216,10 +218,21 @@
 	(if boldface?
 	    (list 
 	     (and foreign-balance 
-		  (gnc:make-html-text (gnc:html-markup-b foreign-balance)))
-	     (and domestic-balance
-		  (gnc:make-html-text (gnc:html-markup-b domestic-balance))))
-	    (list foreign-balance domestic-balance))
+		  (gnc:make-html-table-cell/markup 
+		   "number-cell"
+		   (gnc:make-html-text (gnc:html-markup-b foreign-balance))))
+	     (and 
+	      domestic-balance
+	      (gnc:make-html-table-cell/markup 
+	       "number-cell"
+	       (gnc:make-html-text (gnc:html-markup-b domestic-balance)))))
+	    (list 
+	     (gnc:make-html-table-cell/markup 
+	      "number-cell"
+	      foreign-balance)
+	     (gnc:make-html-table-cell/markup 
+	      "number-cell"
+	      domestic-balance)))
 	(gnc:html-make-empty-cells (* 2 (- current-depth 
 					   (if group-header-line? 0 1)))))))
     
@@ -235,15 +248,17 @@
 	(commodity-row-helper! 
 	 my-name #f
 	 (if balance 
-	     (balance 'getmonetary report-commodity reverse-balance?)
+	     (gnc:commodity-collector-assoc 
+	      balance report-commodity reverse-balance?)
 	     #f)
 	 main-row-style)
 	;; Special case for stock-accounts: then the foreign commodity
 	;; gets displayed in this line rather then the following lines
 	;; (loop below). Is also used if is-stock-account? is true.
 	(let ((my-balance 
-	       (if balance (balance 'getmonetary 
-				    my-commodity reverse-balance?) #f)))
+	       (if balance 
+		   (gnc:commodity-collector-assoc 
+		    balance my-commodity reverse-balance?) #f)))
 	  (set! already-printed my-commodity)
 	  (commodity-row-helper! 
 	   my-name
@@ -255,8 +270,8 @@
     ;; balance and its corresponding value in the
     ;; report-currency. One row for each non-report-currency. 
     (if (and balance (not is-stock-account?))
-	(balance 
-	 'format 
+	(gnc:commodity-collector-map
+	 balance 
 	 (lambda (curr val)
 	   (if (or (gnc:commodity-equiv? curr report-commodity)
 		   (and already-printed
@@ -274,7 +289,7 @@
 		  bal
 		  (exchange-fn bal report-commodity)
 		  other-rows-style))))
-	 #f))))
+	 ))))
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -348,7 +363,7 @@
 	 show-col-headers?
 	 show-total? get-total-fn
 	 total-name group-types? show-parent-balance? show-parent-total? 
-	 show-other-curr? report-commodity exchange-fn)
+	 show-other-curr? report-commodity exchange-fn show-zero-entries?)
   (let ((table (gnc:make-html-table))
 	(topl-accounts (gnc:group-get-account-list 
 			(gnc:get-current-group))))
@@ -372,32 +387,42 @@
     ;; Additional function that includes the subaccounts as
     ;; well. Note: It is necessary to define this here (instead of
     ;; changing an argument for account-get-balance) because the
-    ;; show-acct? query is needed.
+    ;; use-acct? query is needed.
     (define (my-get-balance account)
       ;; this-collector for storing the result
       (let ((this-collector (my-get-balance-nosub account)))
 	(for-each 
 	 (lambda (x) (if x 
-			 (this-collector 'merge x #f)))
+			 (gnc:commodity-collector-merge 
+			  this-collector x )))
 	 (gnc:group-map-all-accounts
 	  (lambda (a)
 	    ;; Important: Calculate the balance if and only of the
-	    ;; account a is shown, i.e. (show-acct? a) == #t.
-	    (and (show-acct? a)
+	    ;; account a is shown, i.e. (use-acct? a) == #t.
+	    (and (use-acct? a)
 		 (my-get-balance-nosub a)))
 	  (gnc:account-get-children account)))
 	this-collector))
 
-    ;; show this account? Check against the account selection and,
-    ;; if not selected, show-subaccts?==#t and any parent was
-    ;; selected. (Maybe the other way around is more effective?)
-    (define (show-acct? a)
+    ;; Use this account in the account hierarchy? Check against the
+    ;; account selection and, if not selected, show-subaccts?==#t and
+    ;; any parent was selected. (Maybe the other way around is more
+    ;; effective?)
+    (define (use-acct? a)
       (or (member a accounts)
 	  (and show-subaccts? 
 	       (let ((parent (gnc:account-get-parent-account a)))
 		 (and parent
 		      (show-acct? parent))))))
 
+    ;; Show this account? Only if nonzero amount or appropriate
+    ;; preference.
+    (define (show-acct? a)
+      (and (or show-zero-entries?
+	       (not (gnc:commodity-collector-allzero? 
+		     (my-get-balance a))))
+	   (use-acct? a)))
+
     ;; sort an account list. Currently this uses only the account-code
     ;; field, but anyone feel free to add more options to this.
     (define (sort-fn accts)
@@ -525,7 +550,7 @@
 				  subaccounts my-get-balance 
 				  gnc:account-reverse-balance?)))
 		 (if thisbalance 
-		     (subbalance 'merge thisbalance #f))
+		     (gnc:commodity-collector-merge subbalance thisbalance))
 		 subbalance)
 	       heading-style
 	       #t #f)))))
@@ -542,11 +567,12 @@
 	  (for-each 
 	   (lambda (acct)
 	     (let ((subaccts (filter 
-			      show-acct?
+			      use-acct?
 			      (gnc:account-get-immediate-subaccounts acct))))
 	       (if (or (= current-depth tree-depth) (null? subaccts))
 		   (begin
-		     (add-account-rows! acct current-depth alternate)
+		     (if (show-acct? acct)
+			 (add-account-rows! acct current-depth alternate))
 		     (set! alternate (not alternate)))
 		   (add-group! current-depth 
 			       (gnc:html-account-anchor acct)
@@ -573,7 +599,7 @@
 	;; No extra grouping. 
 	;; FIXME: go through accounts even if not
 	;; shown, because the children might be shown.
-	(traverse-accounts! (filter show-acct? topl-accounts) 1))
+	(traverse-accounts! (filter use-acct? topl-accounts) 1))
 
     (remove-last-empty-row)
 
@@ -584,7 +610,7 @@
 	   table "grand-total" (* (if show-other-curr? 3 2) tree-depth))
           (add-subtotal-row! 
            1 total-name 
-           (get-total-fn (filter show-acct? topl-accounts) my-get-balance)
+           (get-total-fn (filter use-acct? topl-accounts) my-get-balance)
 	   "grand-total"
            #t #f)))
     
@@ -672,7 +698,8 @@
     
     table))
 
-(define (gnc:html-make-no-account-warning report-title-string)
+(define (gnc:html-make-no-account-warning
+	 report-title-string report-id)
   (let ((p (gnc:make-html-text)))
     (gnc:html-text-append! 
      p 
@@ -682,9 +709,17 @@
      (gnc:html-markup-h2 (_ "No accounts selected"))
      (gnc:html-markup-p
       (_ "This report requires accounts to be selected.")))
+    (if report-id
+	(gnc:html-text-append! 
+	 p 
+	 (gnc:html-markup-p
+	  (gnc:html-markup-anchor
+	   (sprintf #f "gnc-options:report-id=%a" report-id)
+	   (_ "Edit report options")))))
     p))
 
-(define (gnc:html-make-empty-data-warning report-title-string)
+(define (gnc:html-make-empty-data-warning 
+	 report-title-string report-id)
   (let ((p (gnc:make-html-text)))
     (gnc:html-text-append! 
      p 
@@ -693,4 +728,11 @@
      (gnc:html-markup-h2 (_ "No data"))
      (gnc:html-markup-p
       (_ "The selected accounts contain no data/transactions (or only zeroes) for the selected time period")))
-    p))
\ No newline at end of file
+    (if report-id
+	(gnc:html-text-append! 
+	 p 
+	 (gnc:html-markup-p
+	  (gnc:html-markup-anchor
+	   (sprintf #f "gnc-options:report-id=%a" report-id)
+	   (_ "Edit report options")))))
+    p))
