@@ -3,7 +3,7 @@
 ;;;  representations for parts of an imported Quicken file.  
 ;;;
 ;;;  Bill Gribble <grib@billgribble.com> 20 Feb 2000 
-;;;  $Id: qif-objects.scm,v 1.11 2000/06/13 19:51:42 peticolas Exp $
+;;;  $Id: qif-objects.scm,v 1.16.2.1 2001/06/20 18:56:25 grib Exp $
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (gnc:support "qif-import/qif-objects.scm")
@@ -12,7 +12,6 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;  qif-file class 
-;;  currency     : a string representing the file's currency unit
 ;;  xtns         : list of <qif-xtn>  
 ;;  accounts     : list of <qif-acct>  
 ;;  cats         : list of <qif-cat>  
@@ -23,10 +22,7 @@
   (make-simple-class 
    'qif-file 
    '(path                 ;; where file was loaded 
-     default-account      ;; guessed or specified default account name
-     default-account-type ;; either GNC-BANK-TYPE or GNC-STOCK-TYPE 
      y2k-threshold
-     currency             ;; this is a string.. no checking 
      xtns                
      accounts 
      cats
@@ -38,18 +34,6 @@
 (define qif-file:path 
   (simple-obj-getter <qif-file> 'path))
 
-(define qif-file:default-account 
-  (simple-obj-getter <qif-file> 'default-account))
-
-(define qif-file:set-default-account!
-  (simple-obj-setter <qif-file> 'default-account))
-
-(define qif-file:default-account-type 
-  (simple-obj-getter <qif-file> 'default-account-type))
-
-(define qif-file:set-default-account-type! 
-  (simple-obj-setter <qif-file> 'default-account-type))
-
 (define qif-file:set-path! 
   (simple-obj-setter <qif-file> 'path))
 
@@ -59,12 +43,6 @@
 (define qif-file:set-y2k-threshold!
   (simple-obj-setter <qif-file> 'y2k-threshold))
 
-(define qif-file:currency 
-  (simple-obj-getter <qif-file> 'currency))
-
-(define qif-file:set-currency!
-  (simple-obj-setter <qif-file> 'currency))
-
 (define qif-file:cats 
   (simple-obj-getter <qif-file> 'cats))
 
@@ -89,10 +67,8 @@
 (define qif-file:set-accounts!
   (simple-obj-setter <qif-file> 'accounts))
 
-(define (make-qif-file account currency) 
+(define (make-qif-file) 
   (let ((self (make-simple-obj <qif-file>)))
-    (qif-file:set-default-account! self account)
-    (qif-file:set-currency! self currency)
     (qif-file:set-y2k-threshold! self 50)
     (qif-file:set-xtns! self '())
     (qif-file:set-accounts! self '())
@@ -108,7 +84,8 @@
 (define <qif-split>
   (make-simple-class 
    'qif-split
-   '(category class memo amount category-is-account? matching-cleared mark)))
+   '(category class memo amount category-is-account? matching-cleared mark
+              miscx-category miscx-is-account? miscx-class)))
 
 (define qif-split:category 
   (simple-obj-getter <qif-split> 'category))
@@ -121,11 +98,17 @@
           (qif-split:parse-category self value))
          (cat-name (list-ref cat-info 0))
          (is-account? (list-ref cat-info 1))
-         (class-name (list-ref cat-info 2)))
+         (class-name (list-ref cat-info 2))
+         (miscx-name (list-ref cat-info 3))
+         (miscx-is-account? (list-ref cat-info 4))
+         (miscx-class (list-ref cat-info 5)))
     (qif-split:set-category-private! self cat-name)
     (qif-split:set-class! self class-name)
-    (qif-split:set-category-is-account?! self is-account?)))
-
+    (qif-split:set-category-is-account?! self is-account?)
+    (qif-split:set-miscx-category! self miscx-name)
+    (qif-split:set-miscx-is-account?! self miscx-is-account?)
+    (qif-split:set-miscx-class! self miscx-class)))
+    
 (define qif-split:class 
   (simple-obj-getter <qif-split> 'class))
 
@@ -162,20 +145,37 @@
 (define qif-split:set-category-is-account?! 
   (simple-obj-setter <qif-split> 'category-is-account?))
 
+(define qif-split:miscx-is-account? 
+  (simple-obj-getter <qif-split> 'miscx-is-account?))
+
+(define qif-split:set-miscx-is-account?!
+  (simple-obj-setter <qif-split> 'miscx-is-account?))
+
+(define qif-split:miscx-category 
+  (simple-obj-getter <qif-split> 'miscx-category))
+
+(define qif-split:set-miscx-category!
+  (simple-obj-setter <qif-split> 'miscx-category))
+
+(define qif-split:miscx-class 
+  (simple-obj-getter <qif-split> 'miscx-class))
+
+(define qif-split:set-miscx-class!
+  (simple-obj-setter <qif-split> 'miscx-class))
+
 (define (make-qif-split)
   (let ((self (make-simple-obj <qif-split>)))
     (qif-split:set-category! self "")
     self))
 
 
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;  qif-xtn class 
 ;;  [D] date       : parsed. 
 ;;  [P] payee      : string 
 ;;  [N] number (check number, sell, or buy)
 ;;  [C] cleared    : parsed (x/X/*) ;
-;;  [T] amount     : parsed, units are currency from <qif-file>. 
+;;  [T] amount     : parsed, units are currency of dest account
 ;;  [I] share price : parsed
 ;;  [Q] number of shares
 ;;  [Y] name of security 
@@ -281,6 +281,7 @@
 (define (qif-xtn:print self)
   (simple-obj-print self))
 
+
 (define (qif-xtn:split-amounts self)
   (map 
    (lambda (split)
@@ -486,3 +487,169 @@
           (set! namestring (string-replace-char! namestring #\_ #\space))
           namestring)
         "QIF Import")))
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; qif-map-entry class
+;; information for mapping a QIF account/category name to a
+;; gnucash name.
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define <qif-map-entry>
+  (make-simple-class
+   'qif-map-entry
+   '(qif-name       ;; set while parsing file 
+     allowed-types  ;; set while parsing file 
+     description    ;; from QIF acct, if there is one 
+     gnc-name       ;; set from guess-map 
+     new-acct?      ;; set from guess-map
+     display?)))    ;; set when non-zero transactions 
+
+(define (make-qif-map-entry)
+  (make-simple-obj <qif-map-entry>))
+
+(define (qif-map-entry:clone orig)
+  (let ((me (make-qif-map-entry)))
+    (qif-map-entry:set-qif-name! me (qif-map-entry:qif-name orig))
+    (qif-map-entry:set-allowed-types! me (qif-map-entry:allowed-types orig))
+    (qif-map-entry:set-description! me (qif-map-entry:description orig))
+    (qif-map-entry:set-gnc-name! me (qif-map-entry:gnc-name orig))
+    (qif-map-entry:set-new-acct?! me (qif-map-entry:new-acct? orig))
+    (qif-map-entry:set-display?! me (qif-map-entry:display? orig))
+    me))
+
+(define (qif-map-entry:allowed-parent-types self) 
+  (let ((types-list (reverse (qif-map-entry:allowed-types self))))
+    (define (add-types . rest)
+      (for-each 
+       (lambda (t)
+         (if (not (memv t types-list))
+             (set! types-list (cons t types-list))))
+       rest))
+    
+    (for-each 
+     (lambda (t)
+       (cond 
+        ((memv t (list GNC-BANK-TYPE GNC-CASH-TYPE GNC-CCARD-TYPE 
+                       GNC-STOCK-TYPE GNC-MUTUAL-TYPE
+                       GNC-ASSET-TYPE GNC-LIABILITY-TYPE))
+         (add-types GNC-BANK-TYPE GNC-CASH-TYPE GNC-CCARD-TYPE 
+                    GNC-STOCK-TYPE GNC-MUTUAL-TYPE
+                    GNC-ASSET-TYPE GNC-LIABILITY-TYPE))
+        ((memv t (list GNC-INCOME-TYPE GNC-EXPENSE-TYPE))
+         (add-types GNC-INCOME-TYPE GNC-EXPENSE-TYPE))
+        (#t
+         (add-types t))))
+     (qif-map-entry:allowed-types self))
+    (reverse types-list)))
+
+
+(define qif-map-entry:qif-name
+  (simple-obj-getter <qif-map-entry> 'qif-name))
+
+(define qif-map-entry:set-qif-name!
+  (simple-obj-setter <qif-map-entry> 'qif-name))
+
+(define qif-map-entry:allowed-types
+  (simple-obj-getter <qif-map-entry> 'allowed-types))
+
+(define qif-map-entry:set-allowed-types!
+  (simple-obj-setter <qif-map-entry> 'allowed-types))
+
+(define qif-map-entry:description
+  (simple-obj-getter <qif-map-entry> 'description))
+
+(define qif-map-entry:set-description!
+  (simple-obj-setter <qif-map-entry> 'description))
+
+(define qif-map-entry:gnc-name
+  (simple-obj-getter <qif-map-entry> 'gnc-name))
+
+(define qif-map-entry:set-gnc-name!
+  (simple-obj-setter <qif-map-entry> 'gnc-name))
+
+(define qif-map-entry:new-acct?
+  (simple-obj-getter <qif-map-entry> 'new-acct?))
+
+(define qif-map-entry:set-new-acct?!
+  (simple-obj-setter <qif-map-entry> 'new-acct?))
+
+(define qif-map-entry:display?
+  (simple-obj-getter <qif-map-entry> 'display?))
+
+(define qif-map-entry:set-display?!
+  (simple-obj-setter <qif-map-entry> 'display?))
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;  <qif-stock-symbol>
+;;  [N] stock name     : string 
+;;  [S] ticker symbol  : string 
+;;  [T] type           : string 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define <qif-stock-symbol>
+  (make-simple-class
+   'qif-stock-symbol
+   '(name symbol type)))
+
+(define qif-stock-symbol:name
+  (simple-obj-getter <qif-stock-symbol> 'name))
+
+(define qif-stock-symbol:set-name! 
+  (simple-obj-setter <qif-stock-symbol> 'name))
+
+(define qif-stock-symbol:symbol
+  (simple-obj-getter <qif-stock-symbol> 'symbol))
+
+(define qif-stock-symbol:set-symbol! 
+  (simple-obj-setter <qif-stock-symbol> 'symbol))
+
+(define qif-stock-symbol:type
+  (simple-obj-getter <qif-stock-symbol> 'type))
+
+(define qif-stock-symbol:set-type! 
+  (simple-obj-setter <qif-stock-symbol> 'type))
+
+(define (qif-stock-symbol:print self)
+  (simple-obj-print self))
+
+(define (make-qif-stock-symbol)
+  (let ((retval (make-simple-obj <qif-stock-symbol>)))
+    (qif-stock-symbol:set-name! retval "")
+    (qif-stock-symbol:set-symbol! retval "")
+    retval))
+
+(define <qif-ticker-map>
+  (make-simple-class
+   'qif-ticker-map
+   '(stocks)))
+
+(define qif-ticker-map:ticker-map
+  (simple-obj-getter <qif-ticker-map> 'stocks))
+
+(define qif-ticker-map:set-ticker-map!
+  (simple-obj-setter <qif-ticker-map> 'stocks))
+
+(define (make-ticker-map) 
+  (let ((self (make-simple-obj <qif-ticker-map>)))
+    (qif-ticker-map:set-ticker-map! self '())
+    self))
+
+(define (qif-ticker-map:add-ticker! ticker-map stock-symbol)
+  (qif-ticker-map:set-ticker-map!
+   ticker-map
+   (cons stock-symbol (qif-ticker-map:ticker-map ticker-map))))
+
+(define (qif-ticker-map:lookup-ticker ticker-map name)
+  (let ((retval #f))
+    (for-each 
+     (lambda (symbol)
+       (if (string=? name (qif-stock-symbol:name symbol))
+	   (begin
+	     (set! retval (qif-stock-symbol:symbol symbol))
+	     (if (string=? retval "")
+		 (set! retval #f)))))
+     (qif-ticker-map:ticker-map ticker-map))
+    retval))
+
