@@ -3,16 +3,16 @@
 ;;;  routines to parse values and dates in QIF files. 
 ;;;
 ;;;  Bill Gribble <grib@billgribble.com> 20 Feb 2000 
-;;;  $Id: qif-parse.scm,v 1.11 2000/06/12 23:38:01 peticolas Exp $
+;;;  $Id: qif-parse.scm,v 1.18.2.1 2001/06/20 18:56:26 grib Exp $
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(gnc:support "qif-import/qif-parse.scm")
+;(gnc:support "qif-import/qif-parse.scm")
 
 (define qif-category-compiled-rexp 
-  (make-regexp "^ *(\\[)?([^]/]*)(]?)(/?)(.*) *$"))
+  (make-regexp "^ *(\\[)?([^]/\\|]*)(]?)(/?)([^\|]*)(\\|(\\[)?([^]/]*)(]?)(/?)(.*))? *$"))
 
 (define qif-date-compiled-rexp 
-  (make-regexp "^ *([0-9]+) *[-/.'] *([0-9]+) *[-/.'] *([0-9]+).*$"))
+  (make-regexp "^ *([0-9]+) *[-/.'] *([0-9]+) *[-/.'] *([0-9]+).*$|^ *([0-9][0-9][0-9][0-9])([0-9][0-9])([0-9][0-9]).*$"))
 
 (define decimal-radix-regexp
   (make-regexp 
@@ -27,24 +27,39 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;  qif-split:parse-category 
 ;;  this one just gets nastier and nastier. 
-;;  ATM we return a list of 3 elements: parsed category name 
-;;  (without [] if it was an account name), bool stating if it 
-;;  was an account name, and string representing the class name 
-;;  (or #f if no class).
+;;  ATM we return a list of 6 elements: 
+;;    parsed category name (without [] if it was an account name)
+;;    bool stating if it was an account name
+;;    class of account or #f 
+;;    string representing the "miscx category" if any 
+;;    bool if miscx category is an account
+;;    class of miscx cat or #f 
 ;;  gosh, I love regular expressions. 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define (qif-split:parse-category self value)
   (let ((match (regexp-exec qif-category-compiled-rexp value)))
     (if match
-        (begin 
-          (list (match:substring match 2)
-                (if (and (match:substring match 1)
-                         (match:substring match 3))
-                    #t #f)
-                (if (match:substring match 4)
-                    (match:substring match 5)
-                    #f)))
+        (let ((rv
+               (list (match:substring match 2)
+                     (if (and (match:substring match 1)
+                              (match:substring match 3))
+                         #t #f)
+                     (if (match:substring match 4)
+                         (match:substring match 5)
+                         #f)
+                     ;; miscx category name 
+                     (if (match:substring match 6)
+                         (match:substring match 8)
+                         #f)
+                     ;; is it an account? 
+                     (if (and (match:substring match 7)
+                              (match:substring match 9))
+                         #t #f)
+                     (if (match:substring match 10)
+                         (match:substring match 11)
+                         #f))))
+          rv)
         (begin 
           (display "qif-split:parse-category : can't parse ")
           (display value) (newline)
@@ -129,40 +144,26 @@
                             (string-remove-leading-space read-value)))))
     (cond
      ((string=? mangled-string "bank")
-      GNC-BANK-TYPE)
+      (list GNC-BANK-TYPE))
      ((string=? mangled-string "port")
-      GNC-BANK-TYPE)
+      (list GNC-BANK-TYPE))
      ((string=? mangled-string "cash")
-      GNC-CASH-TYPE)
+      (list GNC-CASH-TYPE))
      ((string=? mangled-string "ccard")
-      GNC-CCARD-TYPE)
+      (list GNC-CCARD-TYPE))
      ((string=? mangled-string "invst") ;; these are brokerage accounts.
-      GNC-BANK-TYPE)
+      (list GNC-BANK-TYPE))
      ((string=? mangled-string "oth a")
-      GNC-ASSET-TYPE)
+      (list GNC-ASSET-TYPE GNC-BANK-TYPE GNC-CASH-TYPE))
      ((string=? mangled-string "oth l")
-      GNC-LIABILITY-TYPE)
+      (list GNC-LIABILITY-TYPE GNC-CCARD-TYPE))
      ((string=? mangled-string "mutual")
-      GNC-MUTUAL-TYPE)
+      (list GNC-BANK-TYPE))
      (#t
       (display "qif-parse:parse-acct-type : unhandled account type ")
       (display read-value)
       (display "... substituting Bank.")
-      GNC-BANK-TYPE))))
-
-(define (qif-parse:state-to-account-type qstate)
-  (cond ((eq? qstate 'type:bank)
-         GNC-BANK-TYPE)
-        ((eq? qstate 'type:cash)
-         GNC-CASH-TYPE)
-        ((eq? qstate 'type:ccard)
-         GNC-CCARD-TYPE)
-        ((eq? qstate 'type:invst) ;; these are brokerage accounts in quicken
-         GNC-BANK-TYPE)        
-        ((eq? qstate '#{type:oth\ a}#)
-         GNC-ASSET-TYPE)
-        ((eq? qstate '#{type:oth\ l}#)
-         GNC-LIABILITY-TYPE)))
+      (list GNC-BANK-TYPE)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;  parse-bang-field : the bang fields switch the parse context for 
@@ -183,57 +184,82 @@
            'buy)
           ((buyx kaufx)
            'buyx)
-          ((sell)  ;; verkaufen
-           'sell)
-          ((sellx)
-           'sellx)
+          ((cglong kapgew) ;; Kapitalgewinnsteuer
+           'cglong)
+          ((cglongx kapgewx)
+           'cglongx)
+          ((cgmid) ;; Kapitalgewinnsteuer
+           'cgmid)
+          ((cgmidx)
+           'cgmidx)
+          ((cgshort k.gewsp)
+           'cgshort)
+          ((cgshortx k.gewspx)
+           'cgshortx)
           ((div)   ;; dividende
            'div) 
           ((divx)    
            'divx)
+;          ((exercise)
+;           'exercise)
+;          ((exercisx)
+;           'exercisx)
+;          ((expire)
+;           'expire)
+;          ((grant)
+;           'grant)
           ((int intinc aktzu) ;; zinsen
            'intinc)
           ((intx intincx)
            'intincx)
-          ((cglong) ;; Kapitalgewinnsteuer
-           'cglong)
-          ((cglongx)
-           'cglongx)
-          ((cgshort)
-           'cgshort)
-          ((cgshortx)
-           'cgshortx)
-          ((shrsin)
-           'shrsin)
-          ((shrsout)
-           'shrsout)
-          ((xin) 
-           'xin)
-          ((xout) 
-           'xout)
-          ((stksplit)
-           'stksplit)
-          ((reinvdiv)
-           'reinvdiv)
-          ((reinvint)
-           'reinvint)
-          ((reinvsg)
-           'reinvsg)
-          ((reinvsh)
-           'reinvsh)
-          ((reinvlg reinvkur)
-           'reinvlg)
-          ((miscinc)
-           'miscinc)
-          ((miscincx)
-           'miscincx)
+          ((margint)
+           'margint)
+          ((margintx)
+           'margintx)
           ((miscexp)
            'miscexp)
           ((miscexpx)
            'miscexpx)
+          ((miscinc)
+           'miscinc)
+          ((miscincx)
+           'miscincx)
+          ((reinvdiv)
+           'reinvdiv)
+          ((reinvint reinvzin)
+           'reinvint)
+          ((reinvlg reinvkur)
+           'reinvlg)
+          ((reinvmd)
+           'reinvmd)
+          ((reinvsg reinvksp)
+           'reinvsg)
+          ((reinvsh)
+           'reinvsh)
+          ((reminder erinnerg)
+           'reminder)
+          ((sell verkauf)  ;; verkaufen
+           'sell)
+          ((sellx verkaufx)
+           'sellx)
+          ((shrsin aktzu)
+           'shrsin)
+          ((shrsout aktab)
+           'shrsout)
+          ((stksplit aktsplit)
+           'stksplit)
+          ((xin) 
+           'xin)
+          ((xout) 
+           'xout)
+;          ((vest) 
+;           'vest)
           (else 
-           (display "qif-parse:parse-action-field : unknown action field ")
-           (write read-value) (newline)
+           (gnc:warning-dialog 
+            (string-append 
+             (sprintf #f "The file contains an unknown Action '%s'.\n"
+                      read-value)
+             "Some transactions may be discarded."))
            #f)))
       #f))
 
@@ -276,10 +302,14 @@
           (match (regexp-exec qif-date-compiled-rexp date-string)))
       
       (if match
-          (set! date-parts (list (match:substring match 1)
-                                 (match:substring match 2)
-                                 (match:substring match 3))))
-      
+          (if (match:substring match 1)
+              (set! date-parts (list (match:substring match 1)
+                                     (match:substring match 2)
+                                     (match:substring match 3)))
+              (set! date-parts (list (match:substring match 4)
+                                     (match:substring match 5)
+                                     (match:substring match 6)))))
+
       ;; get the strings into numbers (but keep the strings around)
       (set! numeric-date-parts
             (map (lambda (elt)
@@ -337,10 +367,13 @@
         (numeric-date-parts '())
         (retval date-string)
         (match (regexp-exec qif-date-compiled-rexp date-string)))
-    (if match
+    (if (match:substring match 1)
         (set! date-parts (list (match:substring match 1)
                                (match:substring match 2)
-                               (match:substring match 3))))
+                               (match:substring match 3)))
+        (set! date-parts (list (match:substring match 4)
+                               (match:substring match 5)
+                               (match:substring match 6))))
     
     ;; get the strings into numbers (but keep the strings around)
     (set! numeric-date-parts
@@ -444,41 +477,52 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;  qif-parse:parse-number/format 
 ;;  assuming we know what the format is, parse the string. 
+;;  returns a gnc-numeric; the denominator is set so as to exactly 
+;;  represent the number
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define (qif-parse:parse-number/format value-string format) 
   (case format 
     ((decimal)
-     (let ((read-val
-            (with-input-from-string 
-                (string-remove-char 
-                 (string-remove-char value-string #\,)
-                 #\$)
-              (lambda () (read)))))
+     (let* ((filtered-string
+             (string-remove-char 
+              (string-remove-char value-string #\,)
+              #\$))
+            (read-val
+             (with-input-from-string filtered-string
+               (lambda () (read)))))
        (if (number? read-val)
-           (+ 0.0 read-val)
-           #f)))
+           (gnc:double-to-gnc-numeric
+            (+ 0.0 read-val) GNC-DENOM-AUTO
+            (logior (GNC-DENOM-SIGFIGS (- (string-length filtered-string) 1))
+                    GNC-RND-ROUND))
+           (gnc:numeric-zero))))
     ((comma)
-     (let ((read-val
-            (with-input-from-string 
-                (string-remove-char 
-                 (string-replace-char! 
-                  (string-remove-char value-string #\.)
-                  #\, #\.)
-                 #\$)
-              (lambda () (read)))))
+     (let* ((filtered-string 
+             (string-remove-char 
+              (string-replace-char! 
+               (string-remove-char value-string #\.)
+               #\, #\.)
+              #\$))             
+            (read-val
+             (with-input-from-string filtered-string
+               (lambda () (read)))))
        (if (number? read-val)
-           (+ 0.0 read-val)
-           #f)))
+           (gnc:double-to-gnc-numeric
+            (+ 0.0 read-val) GNC-DENOM-AUTO
+            (logior (GNC-DENOM-SIGFIGS (- (string-length filtered-string) 1))
+                    GNC-RND-ROUND))
+           (gnc:numeric-zero))))
     ((integer)
      (let ((read-val
             (with-input-from-string 
                 (string-remove-char value-string #\$)
               (lambda () (read)))))
        (if (number? read-val)
-           (+ 0.0 read-val)
-           #f)))))
-     
+           (gnc:double-to-gnc-numeric
+            (+ 0.0 read-val) 1 GNC-RND-ROUND)
+           (gnc:numeric-zero))))))
+
 (define (qif-parse:check-number-formats amt-strings formats)
   (let ((retval formats))
     (for-each 
@@ -500,7 +544,7 @@
                    (if (not tmp)
                        (set! all-ok #f))
                    tmp)
-                 0.0))
+                 (gnc:numeric-zero)))
            amt-strings)))
     (if all-ok parsed #f)))
 
@@ -520,6 +564,3 @@
   (with-output-to-string 
     (lambda ()
       (write num))))
-
-(define (qif-parse:print-acct-type t)
-  (symbol->string (gnc:account-type->symbol t)))
\ No newline at end of file
